/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 2010,2011,2012
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 

/*
 * RTC/Jazz change log
 * 
 * 
 * 15 Jul 2013 Shilpa 9.1.0.5 WI 52348
 * Added new Class for Optim Report Intiative instantiated for each Action
 *
 *
*/
/*
 * File: OptimReportingContext.cpp
 *
 * Function: Main class for OptimReportingContext Generation
 *
 * Initial revision.
*/
#include "RtStdAfx.h"       // Required for preprocess header

#define  OptimReportingContext_CPP       // Tell include ok to export
#include "RtCommon.h"       // Our common includes
#include "RtServ.h"         // RtServGetPstDirectory
#include "OptimReportingContext.h"       // Our header

#include "RtTable.h"
#include "RtAudit.h"        // RtAudit

#include "ActnPort.h"       // Action Results portable Class
#include <ENVCGLBL.H>       // Environment Globals

int outputReport(OptimReportingContext_t *context);

/*----------------------------------------------------------------------------
 * Copyright and fix list
 *---------------------------------------------------------------------------*/
RtCopyright();

// ---------------------------------------------------------------------------
#define AC_TRUE _T("1")
#define AC_NA  _T("-1")
#define ORCLINDXROWOVERHEAD 3 // 1 byte for each column in the index, plus 2 
#define DB2INDXROWOVERHEAD  7 // DB2 Index row overhead (minimum overhead irrespective of index type
// ---------------------------------------------------------------------------

/*----------------------------------------------------------------------------
 * OptimReportingContext::OptimReportingContext
 *---------------------------------------------------------------------------*/
OptimReportingContext::OptimReportingContext()
{
   RtTraceEntry(OptimReportingContext::OptimReportingContext);   
   mNumTables   = 0; 
   mFileName = _T("");
   mHtmlFileName = _T("");   
   mCompression = _T("");
   mDBMSType    = _T("");
   mDBMSVer     = _T(""); 
   mSQLText =  _T("");
   mRelName = _T("");
   mFullName = _T("");
   mConnMode = _T("");   
   mpContext    = NULL;   
   OptimReportingContext_t *reportingContext;
   reportingContext = new OptimReportingContext_t;
   mpContext = reportingContext;   
}

/*----------------------------------------------------------------------------
 * OptimReportingContext::~OptimReportingContext
 *---------------------------------------------------------------------------*/
OptimReportingContext::~OptimReportingContext()
{
    RtTraceEntry(OptimReportingContext::~OptimReportingContext);

    mNumTables   = 0;
    mFileName = _T("");
    mHtmlFileName = _T("");    
    mCompression = _T("");
    mDBMSType    = _T("");
    mDBMSVer     = _T("");  
    mRequest     = FALSE;    
    mSQLText =  _T("");    
    mRelName = _T("");
    mFullName = _T("");
    mConnMode = _T("");    
    mpContext->pMessages = NULL;
    mpContext->pRequestOptions = NULL;
    mpContext->pExtendedObjects = NULL;
    mpContext->pEntities = NULL;
    mpContext->pAnalysis = NULL;
    mpContext->pJoinControl = NULL;    
    mpContext->pRelationships = NULL;
    mpContext->pStatistics = NULL; 
    mpContext    = NULL;
}

////////////////////////////////////////////////////////////////////////////////////////
// BOOL OptimReportingContext::BuildOptimReportingContext(CString csObjectData)
//
// Input:   Actnport: Setup Actnport to build reporting structures
// Output:  None.
// Returns: Success or failure.
// Effect:  
// Remarks: Called by ActnRept
//
BOOL OptimReportingContext::BuildOptimReportingContext(ActnPort *pActnPort, int ActionType, CString ActionName, 
                                                       BOOL bRequest, CString ParAction)
{
  RtTraceEntry(OptimReportingContext::BuildOptimReportingContext);
   
  BOOL bSuccess  = FALSE;
  const TCHAR *mode;
  OptimReportingContext_t reportingContext;
  mpContext = &reportingContext;  
  RtDirectory dir(RtDirectory::PstData);
     
  // Setup ActnPort ptr
  mpActnPort = pActnPort;
  mFileName = ORC_REPORT_NAME;

  mActionType = mpActnPort->GetActionType ();
  
  if (!ActionName.IsEmpty())
      mActionName = ActionName;   

   if (ActionName == RtString(RTACTION_ARCHIVE_ACTIONNAME))
   {
      mActionType = Archive;
   }
   
   // 2 request
   mRequest = bRequest;
   
   if (mRequest)
   {
      // already have file name in mFileName
      mFileName += ORC_DELIM_SCORE;
      mFileName += ParAction;         
   }
   else
   {
      mFileName += ORC_DELIM_SCORE;
      mFileName += ActionName;
   } 
   mHtmlFileName = mFileName;

   #ifdef __unix
     mFileName += _T(".xml");
     mHtmlFileName += _T(".html");
   #else
     mFileName += _T(".XML");
     mHtmlFileName += _T(".HTML");      
   #endif
  

   dir.SetPath(mFileName, RtDirectory::PstData);

   // Create the RtUnciode file 
   mode = _T("w");
   if (mRequest)
      mode = _T("a+");    
  
   if (dir.GetExtPart().IsEmpty())
   {
      #ifdef __unix
         dir.SetExtPart(_T("xml"));
      #else
         dir.SetExtPart(_T("XML"));
      #endif
   }    
   
   // Start building Structs for OptimReportingContext Reports
   bSuccess = PopulateOptimReportingContext ();  
   bSuccess = outputReport(mpContext);

   // Write to Optim Directory 
   /*
   if(WriteOptimDirectory() != bSuccess)
   {
      return (FALSE);     
   } 
   */
  
   return (TRUE);
}

///////////////////////////////////////////////////////////////////////////////////////
// BOOL OptimReportingContext::PopulateOptimReportingContext(OptimReporting Context)
//
// Input:   Reporting Structure build the struct to pass to JNI
// Output:  None.
// Returns: Success or failure.
// Effect:  
// Remarks: Called by JNI method
//
BOOL OptimReportingContext::PopulateOptimReportingContext()
{
   RtTraceEntry(OptimReportingContext::PopulateOptimReportingContext);
   
   BOOL bSuccess  = FALSE;
   
   bSuccess = BuildCommonSection();    
   bSuccess = BuildEntity();
   
   switch (mActionType)
   {
      // Convert
      case  ConvertAction:
            mActionName = RtString(RTACTION_CONVERT_ACTIONNAME);            
            bSuccess = BuildJoinControl();         
            break;
      // Compare
      case  CompareAction:
            mActionName = RtString(RTACTION_COMPARE_ACTIONNAME);           
            break;
      // Delete
      case  DeleteAction:
            mActionName = RtString(RTACTION_DELETE_ACTIONNAME);            
            bSuccess = BuildExtendedObjects();
            bSuccess = BuildImpactAnalysis();
            bSuccess = BuildRelationship();
            bSuccess = BuildStatistics();
            break;
      // Archive/Extract
      case  ArchiveAction:
      case  ExtractAction:
            mActionName = (mActionType == Archive ? 
                          RtString(RTACTION_ARCHIVE_ACTIONNAME) :
                          RtString(RTACTION_EXTRACT_ACTIONNAME));            
            bSuccess = BuildExtendedObjects();
            bSuccess = BuildRelationship();
            bSuccess = BuildStatistics();
            break;
      // Insert/Load
      case  InsertAction:
      case  LoadAction:
            mActionName = (mActionType == Insert ? 
                          RtString(RTACTION_INSERT_ACTIONNAME) :
                          RtString(RTACTION_LOAD_ACTIONNAME));                        
            break;
      // Restore
      case  RestoreAction:                         
            break;
      // Report
      case  ReportAction:
            mActionName = RtString(RTACTION_REPORT_ACTIONNAME);                      
            break;
      // Import
      case  Import:
            mActionName = RtString(RTDEFS_STRING_IMPORT);            
            break;

      default:
            break;
   }

   return (TRUE);
}

/*----------------------------------------------------------------------------
 * OptimReportingContext::WriteOptimDirectory
 *    WriteOptimDirectory 
 *---------------------------------------------------------------------------*/
BOOL  OptimReportingContext::WriteOptimDirectory()
{
   RtTraceEntry(BuildOptimReportingContext::WriteOptimDirectory);

   RtAuditXML    AuditRec;  
   CString       CurValue;
   BOOL          bSuccess  = FALSE;
   mORCRec = _T("");


   if (mORCFile.Open(mFileName, _T("r")))
   {
      while(mORCFile.ReadLine(CurValue))
      {
         mORCRec += CurValue;
      }
   }

   // Write to both the tables
   // optim_audit_record and optim_transaction_content
   //if(AuditRec.WriteRec2OptDir (mORCRec) != bSuccess)
   //    return (FALSE); 

   mORCFile.Close();

   return TRUE;   
}

/*---------------------------------------------------------------------------
 * OptimReportingContext:BuildCommonSection
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::BuildCommonSection()
{
   RtTraceEntry(OptimReportingContext::BuildCommonSection);
   
   long StatSteps  = mpActnPort->GetStatStepCount();      
   RtDateTime rtdtStartTime;    // The start time of the process
   RtDateTime rtdtFinishTime;   // The finish time of the process
   RtDateTime rtdtElapsedTime;   // The Elapsed time of the process
   long long lValue = 0;
   long long timeElapsed = 0;
   CString csWord = _T("");  
   BOOL bSuccess = FALSE;   
     
   
   if (StatSteps > 0) 
   {     
       for (int LocCnt = 1 ; LocCnt <= 1 ; LocCnt++) 
       {
         mpActnPort->PrimeStatStep(LocCnt);
         RtRDBA  dba;
         UCHAR Type = _ttoi(mpActnPort->GetStatStepDBMSType());
         csWord  = dba.GetDBTypeLiteral(Type);
         mDBMSType = csWord;
         mDBMSVer = mpActnPort->GetStatStepDBMSVerStr();
         // Process time         
         timeElapsed =  _ttol(mpActnPort->GetStatAccessElapsedTime());         
       }
   }

   if (mDBMSType.IsEmpty())
      GetOptimDBInfo();
   
   //Set Common Structure
   mpContext->errorCount   =   mpActnPort->GetErrorMsgCount();                
   mpContext->warningCount =  mpActnPort->GetWarnMsgCount();             
   mpContext->infoCount = mpActnPort->GetInfoMsgCount();  
   rtdtStartTime.AcceptPortable(mpActnPort->GetStartTime());
   rtdtFinishTime.AcceptPortable(mpActnPort->GetFinishTime());   
   (RtDateTime::FormatDefault);
   if (timeElapsed == 0)
   {
      rtdtElapsedTime.SetDuration(&rtdtStartTime, &rtdtFinishTime); 
      lValue = _ttol((const TCHAR *)rtdtElapsedTime.GetDisplayDuration());
      mpContext->timeElapsed = lValue;
   }
   else
   {
      mpContext->timeElapsed = timeElapsed;
   }
   lValue = _ttol((const TCHAR *)mpActnPort->GetRowsProcessed());
   mpContext->totalRows = lValue;
   lValue = _ttol((const TCHAR *)mpActnPort->GetStrtTbl1stPass());
   mpContext->totalRowsFirstPass = lValue;
   lValue = _ttol((const TCHAR *)mpActnPort->GetTotalErrors());
   mpContext->totalRowsInError = lValue;
   lValue = _ttol((const TCHAR *)mpActnPort->GetTableCount());
   mpContext->totalTables = lValue;   
   wcscpy(mpContext->reportFile, mFileName);
   wcscpy(mpContext->htmlReportFile, mHtmlFileName);  
   wcscpy(mpContext->optimDirectoryName, RtServGetPstDirectory());
	wcscpy(mpContext->optimDirectoryDBMSType, mDBMSType);   
   wcscpy(mpContext->optimDirectoryDBMSVendor, mDBMSType);                     
   wcscpy(mpContext->optimDirectoryDBMSVersion, mDBMSVer);                                          
   wcscpy(mpContext->optimSolutionName, ORC_SOLUTION_NAME);                                      
   wcscpy(mpContext->optimSolutionVersion, ORC_SOLUTION_VERSION);  
   wcscpy( mpContext->optimRuntimeName, ORC_RUNTIME_NAME);                                   
   wcscpy(mpContext->optimRuntimeVersion, ORC_RUNTIME_VERSION);  
   wcscpy(mpContext->clientUserId, mpActnPort->GetClientUserId ());  
   wcscpy(mpContext->requestName, (mpActnPort->GetRequestName().IsEmpty() ? 
                                  RtString(RTRES_UNTITLED): mpActnPort->GetRequestName()));  
   wcscpy(mpContext->serverName, (mpActnPort->GetServerName().IsEmpty() ?
                                 _T("LOCAL"): mpActnPort->GetServerName()));  
   wcscpy(mpContext->serverUserId, mpActnPort->GetServerUserId ());   
   wcscpy(mpContext->timeStarted, rtdtStartTime.GetDisplayDateTime());   
   wcscpy(mpContext->timeFinished, rtdtFinishTime.GetDisplayDateTime());
   (RtDateTime::FormatDefault);        
   switch (mActionType)
   {
        case ArchiveAction:                  
           wcscpy(mpContext->accessDefinitionName, (mpActnPort->GetADName().IsEmpty() ? 
                                                  _T("LOCAL") : mpActnPort->GetADName()));                     
   	     wcscpy(mpContext->accessDefinitionModificationDate, ORC_EMPTY_LSTRING);
           wcscpy(mpContext->extractFile, mpActnPort->GetDestXFName()); 
           wcscpy(mpContext->compareFile, ORC_EMPTY_LSTRING); 
           wcscpy(mpContext->controlFile, ORC_EMPTY_LSTRING); 
           break;

       case ConvertAction:
            wcscpy(mpContext->accessDefinitionName, (mpActnPort->GetADName().IsEmpty() ? 
                                                  _T("LOCAL") : mpActnPort->GetADName()));                     
   	      wcscpy(mpContext->accessDefinitionModificationDate, ORC_EMPTY_LSTRING);    
            wcscpy(mpContext->extractFile,mpActnPort->GetSourceXFName());
	         wcscpy(mpContext->compareFile, ORC_EMPTY_LSTRING); 
            wcscpy(mpContext->controlFile,  mpActnPort->GetControlFileName());        
            break;
         
      case CompareAction:
	        wcscpy(mpContext->accessDefinitionName, (mpActnPort->GetADName().IsEmpty() ? 
                                                           _T("LOCAL") : mpActnPort->GetADName()));
	        wcscpy(mpContext->accessDefinitionModificationDate, ORC_EMPTY_LSTRING);                  
           wcscpy(mpContext->extractFile,mpActnPort->GetSourceXFName());
           wcscpy(mpContext->compareFile, mpActnPort->GetDestXFName());
           wcscpy(mpContext->controlFile,  mpActnPort->GetControlFileName());               
           break;

     case DeleteAction:
          wcscpy(mpContext->accessDefinitionName, (mpActnPort->GetADName().IsEmpty() ? 
                                                  _T("LOCAL") : mpActnPort->GetADName()));
	       wcscpy(mpContext->accessDefinitionModificationDate, ORC_EMPTY_LSTRING); 
          wcscpy(mpContext->extractFile,mpActnPort->GetSourceXFName());
	       wcscpy(mpContext->compareFile, ORC_EMPTY_LSTRING);
          wcscpy(mpContext->controlFile,  mpActnPort->GetControlFileName()); 
          break;

     case ExtractAction:
         wcscpy(mpContext->accessDefinitionName, (mpActnPort->GetADName().IsEmpty() ? 
                                                 _T("LOCAL") : mpActnPort->GetADName()));
         wcscpy(mpContext->accessDefinitionModificationDate, ORC_EMPTY_LSTRING);         
         wcscpy(mpContext->extractFile, mpActnPort->GetDestXFName());
         wcscpy(mpContext->compareFile, ORC_EMPTY_LSTRING); 
         wcscpy(mpContext->controlFile, ORC_EMPTY_LSTRING);
         break;

      case Import:
         wcscpy(mpContext->accessDefinitionName, (mpActnPort->GetADName().IsEmpty() ? 
                                                  _T("LOCAL") : mpActnPort->GetADName()));
         wcscpy(mpContext->accessDefinitionModificationDate, ORC_EMPTY_LSTRING);         
         wcscpy(mpContext->extractFile, ORC_EMPTY_LSTRING);
         wcscpy(mpContext->compareFile, ORC_EMPTY_LSTRING); 
         wcscpy(mpContext->controlFile, ORC_EMPTY_LSTRING);
         break;

      case InsertAction:             
        wcscpy(mpContext->accessDefinitionName, (mpActnPort->GetADName().IsEmpty() ? 
                                                  _T("LOCAL") : mpActnPort->GetADName()));
        wcscpy(mpContext->accessDefinitionModificationDate,L"07-31-2013 13:14:00");
        wcscpy(mpContext->extractFile,mpActnPort->GetSourceXFName());
	     wcscpy(mpContext->compareFile, ORC_EMPTY_LSTRING);
        wcscpy(mpContext->controlFile,  mpActnPort->GetControlFileName());        
        break;

       case LoadAction:        
         wcscpy(mpContext->accessDefinitionName, (mpActnPort->GetADName().IsEmpty() ? 
                                                  _T("LOCAL") : mpActnPort->GetADName()));
        wcscpy(mpContext->accessDefinitionModificationDate,L"07-31-2013 13:14:00");
        wcscpy(mpContext->extractFile,mpActnPort->GetSourceXFName());
	     wcscpy(mpContext->compareFile, ORC_EMPTY_LSTRING);
        wcscpy(mpContext->controlFile,  mpActnPort->GetControlFileName());       
        break;

      case ReportAction:
         wcscpy(mpContext->accessDefinitionName, (mpActnPort->GetADName().IsEmpty() ? 
                                                  _T("LOCAL") : mpActnPort->GetADName()));
         wcscpy(mpContext->accessDefinitionModificationDate, ORC_EMPTY_LSTRING); 
         wcscpy(mpContext->extractFile,mpActnPort->GetSourceXFName());
	      wcscpy(mpContext->compareFile, ORC_EMPTY_LSTRING);
         wcscpy(mpContext->controlFile,  mpActnPort->GetControlFileName());         
         break;

      case RestoreAction:
         wcscpy(mpContext->accessDefinitionName, (mpActnPort->GetADName().IsEmpty() ? 
                                                  _T("LOCAL") : mpActnPort->GetADName()));
         wcscpy(mpContext->accessDefinitionModificationDate, ORC_EMPTY_LSTRING); 
         wcscpy(mpContext->extractFile,ORC_EMPTY_LSTRING);
	      wcscpy(mpContext->compareFile, ORC_EMPTY_LSTRING);
         wcscpy(mpContext->controlFile,  ORC_EMPTY_LSTRING);
         break;      

      default:
         wcscpy(mpContext->accessDefinitionName, ORC_EMPTY_LSTRING);
         wcscpy(mpContext->accessDefinitionModificationDate, ORC_EMPTY_LSTRING); 
         wcscpy(mpContext->extractFile,ORC_EMPTY_LSTRING);
	      wcscpy(mpContext->compareFile, ORC_EMPTY_LSTRING);
         wcscpy(mpContext->controlFile,  ORC_EMPTY_LSTRING);
         break;
   }
   mpContext->pMessages = 0;
   mpContext->pRequestOptions = 0;
   mpContext->pExtendedObjects = 0;
   mpContext->pEntities = 0;
   mpContext->pAnalysis = 0;
   mpContext->pJoinControl = 0;    
   mpContext->pRelationships = 0;
   mpContext->pStatistics = 0; 
   mpContext->numMessages  = ORC_DEFAULT_VALUE;  
   mpContext->numRequestOptions  = ORC_DEFAULT_VALUE;         
   mpContext->numExtendedObjects  = ORC_DEFAULT_VALUE;       
   mpContext->numEntities  = ORC_DEFAULT_VALUE;               
   mpContext->numImpactAnalysis  = ORC_DEFAULT_VALUE;         
   mpContext->numRelationships  = ORC_DEFAULT_VALUE;          
   mpContext->numStatistics  = ORC_DEFAULT_VALUE;           
   mpContext->numJoinControl  = ORC_DEFAULT_VALUE;
   switch (mActionType)
   {
      case ArchiveAction:
         mpContext->accessDefinitionType =  ORC_DEFAULT_VALUE;
         mpContext->reportType = OptimReportingContext::Archive; 
         wcscpy(mpContext->ddl, ORC_EMPTY_LSTRING);                                     
         wcscpy(mpContext->tableMapName, ORC_EMPTY_LSTRING);                            
         wcscpy(mpContext->dataSource1, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->dataSource2, ORC_EMPTY_LSTRING);                             
         mpContext->nonEqualSets = ORC_DEFAULT_VALUE;                                
         mpContext->singleTable = TRUE;                                
         mpContext->sourceType = ORC_DEFAULT_VALUE;                                  
         mpContext->sourceType2 = ORC_DEFAULT_VALUE;                                 
         mpContext->totalSets = ORC_DEFAULT_VALUE;                                   
         mpContext->numberOfArchiveFiles = ORC_DEFAULT_VALUE;                        
         mpContext->numberOfSubsetFiles  = ORC_DEFAULT_VALUE;                        
         mpContext->numberOfFilesRestored  = ORC_DEFAULT_VALUE;                       
         wcscpy(mpContext->reportName, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->reportRole, ORC_EMPTY_LSTRING);                              
         wcscpy(mpContext->reportState, ORC_EMPTY_LSTRING);                             
         mpContext->importImported = FALSE;   
         wcscpy(mpContext->fileName, mpActnPort->GetDestXFName()); 
         mpContext->fileSize = _ttol((const TCHAR *)mpActnPort->GetSrcOneOutputDataBytes());            
         break;

       case ConvertAction:
          mpContext->accessDefinitionType =  ORC_DEFAULT_VALUE;
          mpContext->reportType = OptimReportingContext::Convert; 
          wcscpy(mpContext->ddl, ORC_EMPTY_LSTRING);                                     
          wcscpy(mpContext->tableMapName, (mpActnPort->GetTableMapName().IsEmpty() ?
                                        _T("TM") : mpActnPort->GetTableMapName()));                            
          wcscpy(mpContext->dataSource1, ORC_EMPTY_LSTRING);                             
          wcscpy(mpContext->dataSource2, ORC_EMPTY_LSTRING);                             
          mpContext->nonEqualSets = ORC_DEFAULT_VALUE;                                
          mpContext->singleTable = TRUE;                                
          mpContext->sourceType = ORC_DEFAULT_VALUE;                                  
          mpContext->sourceType2 = ORC_DEFAULT_VALUE;                                 
          mpContext->totalSets = ORC_DEFAULT_VALUE;                                   
          mpContext->numberOfArchiveFiles = ORC_DEFAULT_VALUE;                        
          mpContext->numberOfSubsetFiles  = ORC_DEFAULT_VALUE;                        
          mpContext->numberOfFilesRestored  = ORC_DEFAULT_VALUE;                       
          wcscpy(mpContext->reportName, ORC_EMPTY_LSTRING);                             
          wcscpy(mpContext->reportRole, ORC_EMPTY_LSTRING);                              
          wcscpy(mpContext->reportState, ORC_EMPTY_LSTRING);                             
          mpContext->importImported = FALSE;   
          wcscpy(mpContext->fileName, mpActnPort->GetDestXFName()); 
          mpContext->fileSize = _ttol((const TCHAR *)mpActnPort->GetSrcOneOutputDataBytes());         
          break;
         
      case CompareAction:
         mpContext->accessDefinitionType =  ORC_DEFAULT_VALUE;
         mpContext->reportType = OptimReportingContext::Compare; 
         wcscpy(mpContext->ddl, ORC_EMPTY_LSTRING);                                     
         wcscpy(mpContext->tableMapName, (mpActnPort->GetTableMapName().IsEmpty() ?
                                        _T("TM") : mpActnPort->GetTableMapName()));
         wcscpy(mpContext->dataSource1, (GetSource1Name(mpActnPort->GetSource1Type())));        
         wcscpy(mpContext->dataSource2, (GetSource2Name(mpActnPort->GetSource2Type())));               
         mpContext->nonEqualSets = mpActnPort->GetNumNonEqualTableSets();
         csWord = (mpActnPort->GetIsSingleTableProcess()  == 1 ? 
                    RtString(RTOVER_STR_YES) : RtString(RTOVER_STR_NO));
         mpContext->singleTable  = FALSE;
         if (csWord == RtString(RTOVER_STR_YES))
            mpContext->singleTable  = TRUE;          
         mpContext->sourceType = GetSourceType(mpActnPort->GetSource1Type(), COMP_SOURCE_ONE);                          
         mpContext->sourceType2 = GetSourceType(mpActnPort->GetSource2Type(), COMP_SOURCE_TWO); 
         mpContext->totalSets = mpActnPort->GetTableSize();                                 
         mpContext->numberOfArchiveFiles = ORC_DEFAULT_VALUE;                        
         mpContext->numberOfSubsetFiles  = ORC_DEFAULT_VALUE;                        
         mpContext->numberOfFilesRestored  = ORC_DEFAULT_VALUE;                       
         wcscpy(mpContext->reportName, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->reportRole, ORC_EMPTY_LSTRING);                              
         wcscpy(mpContext->reportState, ORC_EMPTY_LSTRING);                             
         mpContext->importImported = FALSE;   
         wcscpy(mpContext->fileName, mpActnPort->GetDestXFName()); 
         mpContext->fileSize = _ttol((const TCHAR *)mpActnPort->GetSrcOneOutputDataBytes());       
        break;

     case DeleteAction:
        mpContext->accessDefinitionType =  ORC_DEFAULT_VALUE;
        mpContext->reportType = OptimReportingContext::Delete; 
        wcscpy(mpContext->ddl, ORC_EMPTY_LSTRING);                                     
        wcscpy(mpContext->tableMapName, (mpActnPort->GetTableMapName().IsEmpty() ?
                                           _T("TM") : mpActnPort->GetTableMapName()));                            
        wcscpy(mpContext->dataSource1, ORC_EMPTY_LSTRING);                             
        wcscpy(mpContext->dataSource2, ORC_EMPTY_LSTRING);                             
        mpContext->nonEqualSets = ORC_DEFAULT_VALUE;                                
        mpContext->singleTable = TRUE;                                
        mpContext->sourceType = ORC_DEFAULT_VALUE;                                  
        mpContext->sourceType2 = ORC_DEFAULT_VALUE;                                 
        mpContext->totalSets = ORC_DEFAULT_VALUE;                                   
        mpContext->numberOfArchiveFiles = ORC_DEFAULT_VALUE;                        
        mpContext->numberOfSubsetFiles  = ORC_DEFAULT_VALUE;                        
        mpContext->numberOfFilesRestored  = ORC_DEFAULT_VALUE;                       
        wcscpy(mpContext->reportName, ORC_EMPTY_LSTRING);                             
        wcscpy(mpContext->reportRole, ORC_EMPTY_LSTRING);                              
        wcscpy(mpContext->reportState, ORC_EMPTY_LSTRING);                             
        mpContext->importImported = FALSE;   
        wcscpy(mpContext->fileName, mpActnPort->GetDestXFName()); 
        mpContext->fileSize = _ttol((const TCHAR *)mpActnPort->GetSrcOneOutputDataBytes());
        break;

     case ExtractAction:
        mpContext->accessDefinitionType =  ORC_DEFAULT_VALUE;
        mpContext->reportType = OptimReportingContext::Extract; 
        wcscpy(mpContext->ddl, ORC_EMPTY_LSTRING);                                     
        wcscpy(mpContext->tableMapName, (mpActnPort->GetTableMapName().IsEmpty() ?
                                           _T("TM") : mpActnPort->GetTableMapName()));                            
        wcscpy(mpContext->dataSource1, ORC_EMPTY_LSTRING);                             
        wcscpy(mpContext->dataSource2, ORC_EMPTY_LSTRING);                             
        mpContext->nonEqualSets = ORC_DEFAULT_VALUE;                                
        mpContext->singleTable = TRUE;                                
        mpContext->sourceType = ORC_DEFAULT_VALUE;                                  
        mpContext->sourceType2 = ORC_DEFAULT_VALUE;                                 
        mpContext->totalSets = ORC_DEFAULT_VALUE;                                   
        mpContext->numberOfArchiveFiles = ORC_DEFAULT_VALUE;                        
        mpContext->numberOfSubsetFiles  = ORC_DEFAULT_VALUE;                        
        mpContext->numberOfFilesRestored  = ORC_DEFAULT_VALUE;                       
        wcscpy(mpContext->reportName, ORC_EMPTY_LSTRING);                             
        wcscpy(mpContext->reportRole, ORC_EMPTY_LSTRING);                              
        wcscpy(mpContext->reportState, ORC_EMPTY_LSTRING);                             
        mpContext->importImported = FALSE;   
        wcscpy(mpContext->fileName, mpActnPort->GetDestXFName()); 
        mpContext->fileSize = _ttol((const TCHAR *)mpActnPort->GetSrcOneOutputDataBytes());
        break;

      case Import:
	      mpContext->accessDefinitionType =  ORC_DEFAULT_VALUE;
         mpContext->reportType = OptimReportingContext::Utility; 
         wcscpy(mpContext->ddl, ORC_EMPTY_LSTRING);                                     
         wcscpy(mpContext->tableMapName, (mpActnPort->GetTableMapName().IsEmpty() ?
                                        _T("TM") : mpActnPort->GetTableMapName()));                            
         wcscpy(mpContext->dataSource1, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->dataSource2, ORC_EMPTY_LSTRING);                             
         mpContext->nonEqualSets = ORC_DEFAULT_VALUE;                                
         mpContext->singleTable = TRUE;                                
         mpContext->sourceType = ORC_DEFAULT_VALUE;                                  
         mpContext->sourceType2 = ORC_DEFAULT_VALUE;                                 
         mpContext->totalSets = ORC_DEFAULT_VALUE;                                   
         mpContext->numberOfArchiveFiles = ORC_DEFAULT_VALUE;                        
         mpContext->numberOfSubsetFiles  = ORC_DEFAULT_VALUE;                        
         mpContext->numberOfFilesRestored  = ORC_DEFAULT_VALUE;                       
         wcscpy(mpContext->reportName, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->reportRole, ORC_EMPTY_LSTRING);                              
         wcscpy(mpContext->reportState, ORC_EMPTY_LSTRING);                             
         mpContext->importImported = TRUE;   
         wcscpy(mpContext->fileName, mpActnPort->GetDestXFName()); 
         mpContext->fileSize = _ttol((const TCHAR *)mpActnPort->GetSrcOneOutputDataBytes());      
         break;

      case InsertAction:   
         mpContext->accessDefinitionType =  ORC_DEFAULT_VALUE;
         mpContext->reportType = OptimReportingContext::Insert; 
         wcscpy(mpContext->ddl,(mpActnPort->GetSQLText().IsEmpty() ? 
                                 _T("NO") : mpActnPort->GetSQLText()));                                     
         wcscpy(mpContext->tableMapName, (mpActnPort->GetTableMapName().IsEmpty() ?
                                        _T("TM") : mpActnPort->GetTableMapName()));                            
         wcscpy(mpContext->dataSource1, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->dataSource2, ORC_EMPTY_LSTRING);                             
         mpContext->nonEqualSets = ORC_DEFAULT_VALUE;                                
         mpContext->singleTable = TRUE;                                
         mpContext->sourceType = ORC_DEFAULT_VALUE;                                  
         mpContext->sourceType2 = ORC_DEFAULT_VALUE;                                 
         mpContext->totalSets = ORC_DEFAULT_VALUE;                                   
         mpContext->numberOfArchiveFiles = ORC_DEFAULT_VALUE;                        
         mpContext->numberOfSubsetFiles  = ORC_DEFAULT_VALUE;                        
         mpContext->numberOfFilesRestored  = ORC_DEFAULT_VALUE;                       
         wcscpy(mpContext->reportName, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->reportRole, ORC_EMPTY_LSTRING);                              
         wcscpy(mpContext->reportState, ORC_EMPTY_LSTRING);                             
         mpContext->importImported = FALSE;   
         wcscpy(mpContext->fileName, mpActnPort->GetSourceXFName()); 
         mpContext->fileSize = _ttol((const TCHAR *)mpActnPort->GetSrcOneOutputDataBytes());    
         break;

       case LoadAction:       
          mpContext->accessDefinitionType =  ORC_DEFAULT_VALUE;
          mpContext->reportType = OptimReportingContext::Load; 
          wcscpy(mpContext->ddl,(mpActnPort->GetSQLText().IsEmpty() ? 
                                 _T("NO") : mpActnPort->GetSQLText()));                                     
          wcscpy(mpContext->tableMapName, (mpActnPort->GetTableMapName().IsEmpty() ?
                                        _T("TM") : mpActnPort->GetTableMapName()));                            
          wcscpy(mpContext->dataSource1, ORC_EMPTY_LSTRING);                             
          wcscpy(mpContext->dataSource2, ORC_EMPTY_LSTRING);                             
          mpContext->nonEqualSets = ORC_DEFAULT_VALUE;                                
          mpContext->singleTable = TRUE;                                
          mpContext->sourceType = ORC_DEFAULT_VALUE;                                  
          mpContext->sourceType2 = ORC_DEFAULT_VALUE;                                 
          mpContext->totalSets = ORC_DEFAULT_VALUE;                                   
          mpContext->numberOfArchiveFiles = ORC_DEFAULT_VALUE;                        
          mpContext->numberOfSubsetFiles  = ORC_DEFAULT_VALUE;                        
          mpContext->numberOfFilesRestored  = ORC_DEFAULT_VALUE;                       
          wcscpy(mpContext->reportName, ORC_EMPTY_LSTRING);                             
          wcscpy(mpContext->reportRole, ORC_EMPTY_LSTRING);                              
          wcscpy(mpContext->reportState, ORC_EMPTY_LSTRING);                             
          mpContext->importImported = FALSE;   
          wcscpy(mpContext->fileName, mpActnPort->GetSourceXFName()); 
          mpContext->fileSize = _ttol((const TCHAR *)mpActnPort->GetSrcOneOutputDataBytes());                                                    
          break;

      case ReportAction:
         mpContext->accessDefinitionType =  ORC_DEFAULT_VALUE;
         mpContext->reportType = OptimReportingContext::Report; 
         wcscpy(mpContext->ddl,(mpActnPort->GetSQLText().IsEmpty() ? 
                                 _T("NO") : mpActnPort->GetSQLText()));                                     
         wcscpy(mpContext->tableMapName, (mpActnPort->GetTableMapName().IsEmpty() ?
                                        _T("TM") : mpActnPort->GetTableMapName()));                            
         wcscpy(mpContext->dataSource1, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->dataSource2, ORC_EMPTY_LSTRING);                             
         mpContext->nonEqualSets = ORC_DEFAULT_VALUE;                                
         mpContext->singleTable = TRUE;                                
         mpContext->sourceType = ORC_DEFAULT_VALUE;                                  
         mpContext->sourceType2 = ORC_DEFAULT_VALUE;                                 
         mpContext->totalSets = ORC_DEFAULT_VALUE;                                   
         mpContext->numberOfArchiveFiles = ORC_DEFAULT_VALUE;                        
         mpContext->numberOfSubsetFiles  = ORC_DEFAULT_VALUE;                        
         mpContext->numberOfFilesRestored  = ORC_DEFAULT_VALUE; 
         //todo                     
         wcscpy(mpContext->reportName, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->reportRole, ORC_EMPTY_LSTRING);                              
         wcscpy(mpContext->reportState, ORC_EMPTY_LSTRING);                             
         mpContext->importImported = FALSE;   
         wcscpy(mpContext->fileName, mpActnPort->GetDestXFName()); 
         mpContext->fileSize = _ttol((const TCHAR *)mpActnPort->GetSrcOneOutputDataBytes());                                                     
         break;

      case RestoreAction:
         mpContext->accessDefinitionType =  ORC_DEFAULT_VALUE;
         mpContext->reportType = OptimReportingContext::Restore; 
         wcscpy(mpContext->ddl,(mpActnPort->GetSQLText().IsEmpty() ? 
                                 _T("NO") : mpActnPort->GetSQLText()));                                     
         wcscpy(mpContext->tableMapName, (mpActnPort->GetTableMapName().IsEmpty() ?
                                        _T("TM") : mpActnPort->GetTableMapName()));                            
         wcscpy(mpContext->dataSource1, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->dataSource2, ORC_EMPTY_LSTRING);                             
         mpContext->nonEqualSets = ORC_DEFAULT_VALUE;                                
         mpContext->singleTable = TRUE;                                
         mpContext->sourceType = ORC_DEFAULT_VALUE;                                  
         mpContext->sourceType2 = ORC_DEFAULT_VALUE;                                 
         mpContext->totalSets = ORC_DEFAULT_VALUE;                                   
         mpContext->numberOfArchiveFiles = _ttol((const TCHAR *)mpActnPort->GetTableCount());                        
         mpContext->numberOfSubsetFiles  = _ttol((const TCHAR *)mpActnPort->GetRowsProcessed());                        
         mpContext->numberOfFilesRestored  = _ttol((const TCHAR *)mpActnPort->GetTotalRows());
                            
         wcscpy(mpContext->reportName, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->reportRole, ORC_EMPTY_LSTRING);                              
         wcscpy(mpContext->reportState, ORC_EMPTY_LSTRING);                             
         mpContext->importImported = FALSE;   
         wcscpy(mpContext->fileName, mpActnPort->GetDestXFName()); 
         mpContext->fileSize = _ttol((const TCHAR *)mpActnPort->GetSrcOneOutputDataBytes());                                                      
         break;      

      default:
         mpContext->accessDefinitionType =  ORC_DEFAULT_VALUE;
         mpContext->reportType =  ORC_DEFAULT_VALUE; 
         wcscpy(mpContext->ddl,ORC_EMPTY_LSTRING);                                     
         wcscpy(mpContext->tableMapName, ORC_EMPTY_LSTRING);                            
         wcscpy(mpContext->dataSource1, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->dataSource2, ORC_EMPTY_LSTRING);                             
         mpContext->nonEqualSets = ORC_DEFAULT_VALUE;                                
         mpContext->singleTable = TRUE;                                
         mpContext->sourceType = ORC_DEFAULT_VALUE;                                  
         mpContext->sourceType2 = ORC_DEFAULT_VALUE;                                 
         mpContext->totalSets = ORC_DEFAULT_VALUE;                                   
         mpContext->numberOfArchiveFiles =  ORC_DEFAULT_VALUE;;                        
         mpContext->numberOfSubsetFiles  =  ORC_DEFAULT_VALUE;;                        
         mpContext->numberOfFilesRestored =  ORC_DEFAULT_VALUE;;
                            
         wcscpy(mpContext->reportName, ORC_EMPTY_LSTRING);                             
         wcscpy(mpContext->reportRole, ORC_EMPTY_LSTRING);                              
         wcscpy(mpContext->reportState, ORC_EMPTY_LSTRING);                             
         mpContext->importImported = FALSE;   
         wcscpy(mpContext->fileName, ORC_EMPTY_LSTRING); 
         mpContext->fileSize = ORC_DEFAULT_VALUE;                                      
         break;
   }
  // mpContext->objectCreationType = 1; 

   //RequestOptions
   bSuccess = BuildRequestOptions();

   //Messages   
   bSuccess = BuildCommonErrorMessages();
   bSuccess = BuildCommonWarnMessages();
   bSuccess = BuildCommonInfoMessages();
   bSuccess = BuildCommonExitMessages(); 
  
   return(TRUE);
}

/*---------------------------------------------------------------------------
 * OptimReportingContext:BuildEntity
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::BuildEntity()
{
   RtTraceEntry(OptimReportingContext::BuildEntity);

   int      ActionType;
   RtTable  rtCSVTable;
   CArray<RtTable *, RtTable *>    arrTables;
   CArray<RtColumn *, RtColumn *>  arrColumns;
   convmcsv *CsvObj;
   long long lValue; 
   OptimReportingEntity_t *reportingEntity;
   reportingEntity = new OptimReportingEntity_t;
   mpContext->pEntities = reportingEntity;
   mpContext->numEntities = mpActnPort->GetTableSize();

   for ( int EntCnt = 1 ; EntCnt <= mpActnPort->GetTableSize(); EntCnt++)
   {
      mpActnPort->PrimeTableInfoMembers(EntCnt); 
      if (EntCnt > 1)
      {
	        reportingEntity->pNext = new OptimReportingEntity_t;
	        reportingEntity = reportingEntity->pNext;
      }     
      if (mpActnPort->GetCurTblTotProcessed().IsEmpty ())
          reportingEntity->recordCount = ORC_DEFAULT_VALUE;
      else
      {
         lValue = _ttol((const TCHAR *)mpActnPort->GetCurTblTotProcessed());
         reportingEntity->recordCount = lValue;
      }
      reportingEntity->compareUnmatched = 0 ;                         
      reportingEntity->compareEqual = ORC_DEFAULT_VALUE; 
      reportingEntity->compareDifferent = ORC_DEFAULT_VALUE;
      reportingEntity->compareDuplicate = ORC_DEFAULT_VALUE; 
      reportingEntity->convertCsvLimitRowCount = ORC_DEFAULT_VALUE;          
      reportingEntity->convertCsvLimitFileSize = ORC_DEFAULT_VALUE;                
      reportingEntity->convertCsvLimitFileCount = ORC_DEFAULT_VALUE;   
      wcscpy(reportingEntity->convertCsvFileName, ORC_EMPTY_LSTRING);
      reportingEntity->insertInserted = ORC_DEFAULT_VALUE;          	                         
      reportingEntity->insertUpdated = ORC_DEFAULT_VALUE;
      if (mpActnPort->GetCurTblTotSkipped().IsEmpty ())
         reportingEntity->skipped = ORC_DEFAULT_VALUE;
      else
      {
         lValue = _ttol((const TCHAR *)mpActnPort->GetCurTblTotSkipped());
         reportingEntity->skipped = lValue;
      }
      if (mpActnPort->GetCurTblTotErr().IsEmpty ())
          reportingEntity->failed = ORC_DEFAULT_VALUE;
      else
      {
        lValue = _ttol((const TCHAR *)mpActnPort->GetCurTblTotErr());
        reportingEntity->failed = lValue;
      }
      reportingEntity->extractAverageRowLength = ORC_DEFAULT_VALUE;  
      reportingEntity->extractMaxLobLength = ORC_DEFAULT_VALUE;
      wcscpy(reportingEntity->status, ORC_EMPTY_LSTRING); //todo status      
      wcscpy(reportingEntity->name, mpActnPort->GetCurTblName());       
      reportingEntity->type = OptimReportingContext::Table;
      reportingEntity->extractCompressionValue = ORC_DEFAULT_VALUE;
      reportingEntity->deleteNotFound = ORC_DEFAULT_VALUE;   
      reportingEntity->extractCompressionEnabled  = FALSE;   
      reportingEntity->extractLob = FALSE;                   	                  
                                                                                       
      ActionType = mpActnPort->GetActionType();

      switch (ActionType)
      {      
         case Compare:
            reportingEntity->compareUnmatched = mpActnPort->GetCurTblCmpUnmatchedRowsS1();   
            reportingEntity->compareEqual = mpActnPort->GetCurTblCmpEqualRows();   
            reportingEntity->compareDifferent = mpActnPort->GetCurTblCmpChangedRows();   
            reportingEntity->compareDuplicate = mpActnPort->GetCurTblCmpNonUniqueRowsS1();
            break;

         case  Convert:
               if (mpActnPort->GetConvertFileType() == CONV_CSV)
               {
                  // Build the lists of tables and columns (we only need the tables)            
                  rtCSVTable.BuildLists(&arrTables, &arrColumns);             
                  for(int index=0; index < CsvObj->mArray.GetSize(); index++)
                  {
                    CsvObj->mGridData = CsvObj->mArray.GetAt(index);
                    if(CsvObj->mGridData.RowCount > 0)            
                        reportingEntity->convertCsvLimitRowCount = CsvObj->mGridData.RowCount;
                    else
                        reportingEntity->convertCsvLimitRowCount = ORC_DEFAULT_VALUE;                 
                    if(CsvObj->mGridData.FileSize > 0)
                        reportingEntity->convertCsvLimitFileSize = CsvObj->mGridData.FileSize;
                    else
                        reportingEntity->convertCsvLimitFileSize = ORC_DEFAULT_VALUE;                   
                    if(CsvObj->mGridData.FileCount > 0)
                       reportingEntity->convertCsvLimitFileCount = CsvObj->mGridData.FileCount;
                    else
                        reportingEntity->convertCsvLimitFileCount = ORC_DEFAULT_VALUE;
                    break;
                  }                  
                  wcscpy(reportingEntity->convertCsvFileName, mpActnPort->GetDestXFName());
               }
               break;

         case Delete:
              lValue = _ttol((const TCHAR *)mpActnPort->GetRowsNotFound());
              reportingEntity->deleteNotFound = lValue;
              break;

         case Extract:              
              reportingEntity->extractCompressionValue = _ttoi(mpActnPort->GetFileCompressRatio());      
              reportingEntity->extractAverageRowLength = ORC_DEFAULT_VALUE;  
              reportingEntity->extractLob = FALSE;
              if (_ttoi(mpActnPort->GetFileCompressRatio()) > 0)
               reportingEntity->extractCompressionEnabled  = TRUE;
              reportingEntity->extractMaxLobLength = ORC_DEFAULT_VALUE; 
            break;

         case  Insert:
               lValue = _ttol((const TCHAR *)mpActnPort->GetCurTblTotProcessed());
               reportingEntity->insertInserted = lValue;
               lValue = _ttol((const TCHAR *)mpActnPort->GetCurTblRowsUpdated()); 
               reportingEntity->insertUpdated = lValue;
               break;

         default:
               break;

      }     
   }
   reportingEntity->pNext = NULL;
   
   return(TRUE);
}

/*---------------------------------------------------------------------------
 * OptimReportingContext:BuildExtendedObjects
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::BuildExtendedObjects()
{
   RtTraceEntry(OptimReportingContext::BuildExtendedObjects);
   
   BOOL bSuccess = FALSE;
   int Index = 0;
   int ExtSize = 18; 
   int iValue = 0;
   long long lValue = 0;
   CString csObject = _T("");
   RtDateTime rtdtFinishTime;   // The finish time of the process   
   OptimReportingNumberPair_t *reportingExtendedObjects;
   reportingExtendedObjects = new OptimReportingNumberPair_t;
   mpContext->pExtendedObjects = reportingExtendedObjects;
   mNumTables = mpActnPort->GetTableSize();   
   
   // Number of Tables
   mExtObject.SetSize(ExtSize);
   mExtType.SetSize(ExtSize);
   Index=0;
   mExtObject[Index]   = mpActnPort->GetObjNumExtTables();
   csObject.Format(_T("%d"), OptimReportingContext::Table);   
   mExtType[Index] = csObject;
   iValue = _ttoi(mpActnPort->GetObjNumExtTables());
   mpContext->numExtendedObjects = iValue;
  
   // Number of Primary Keys
   Index++;
   if (mpActnPort->GetExtObjPrimaryKey())
     mExtObject[Index]   = mpActnPort->GetObjNumExtPrimaryKeys();
   else
     mExtObject[Index] = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::PrimaryKey);   
   mExtType[Index] = csObject;
  
  // Number of Relationships
   Index++;
  if (mpActnPort->GetExtObjRelationship())
     mExtObject[Index] = mpActnPort->GetObjNumExtRelationships();
  else
    mExtObject[Index] = _T("0"); 
  csObject.Format(_T("%d"), OptimReportingContext::Relationship);   
  mExtType[Index] = csObject;
  
  // Number of Indices
  Index++;
  if (mpActnPort->GetExtObjIndex())
   mExtObject[Index]   = mpActnPort->GetObjNumExtIndices();
  else
   mExtObject[Index]   = _T("0");
  csObject.Format(_T("%d"), OptimReportingContext::Index);   
  mExtType[Index] = csObject;
   
   // Number of Aliases
   Index++;
   if (mpActnPort->GetExtObjSynonymAlias())
     mExtObject[Index]  = mpActnPort->GetObjNumExtAliases();
   else
      mExtObject[Index] = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::Alias);   
   mExtType[Index] = csObject;
   
   // Number of Functions
   Index++;
   if (mpActnPort->GetExtObjFunction())
      mExtObject[Index] = mpActnPort->GetObjNumExtFunctions();
   else
      mExtObject[Index] = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::Function);   
   mExtType[Index] = csObject;
 
    
   // Number of Packages
   Index++;
   if (mpActnPort->GetExtObjPackage())
     mExtObject[Index] = mpActnPort->GetObjNumExtPackages();
   else
      mExtObject[Index] = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::Package);   
   mExtType[Index] = csObject;
       
   // Number of Procedures
   Index++;
   if (mpActnPort->GetExtObjProcedure())
      mExtObject[Index] = mpActnPort->GetObjNumExtProcedures();
   else
      mExtObject[Index] = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::Procedure);   
   mExtType[Index] = csObject;
         
   // Number of Sequences
   Index++;
   if (mpActnPort->GetExtObjSequence())
         mExtObject[Index] = mpActnPort->GetObjNumExtSequences();
   else
        mExtObject[Index] = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::Sequence);   
   mExtType[Index] = csObject;
   
   // Number of Triggers
   Index++;
   if (mpActnPort->GetExtObjTrigger())
        mExtObject[Index] = mpActnPort->GetObjNumExtTriggers();
   else
         mExtObject[Index]   = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::Trigger);   
   mExtType[Index] = csObject;
          
   // Number of Views
   Index++;
   if (mpActnPort->GetExtObjView())
        mExtObject[Index] = mpActnPort->GetObjNumExtViews();
   else
        mExtObject[Index] = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::View);   
   mExtType[Index] = csObject;
   
   // Number of Defaults
   Index++;
   if (mpActnPort->GetExtObjDefault())
    	mExtObject[Index] = mpActnPort->GetObjNumExtDefaults();
   else
     	mExtObject[Index]= _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::Default);   
   mExtType[Index] = csObject;
   
   // Number of Rules
   Index++;
   if (mpActnPort->GetExtObjRule())
     	mExtObject[Index] = mpActnPort->GetObjNumExtRules();
   else
     	mExtObject[Index]   = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::Rule);   
   mExtType[Index] = csObject;
   
   // Number of UDTs   
   Index++;
   if (mpActnPort->GetExtObjUDT())
    	mExtObject[Index] = mpActnPort->GetObjNumExtUDTs();
   else
     	mExtObject[Index]   = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::Udt);   
   mExtType[Index] = csObject;
     
   // Number of Assembliec 
   Index++;
   if (mpActnPort->GetExtObjAssembly())
     	mExtObject[Index] = mpActnPort->GetObjNumExtAssemblies();
   else
     	mExtObject[Index]  = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::Assembly);   
   mExtType[Index] = csObject;
     
   // Number of Partition Functions
   Index++;
   if (mpActnPort->GetExtObjParFunc())
     	mExtObject[Index]= mpActnPort->GetObjNumExtParFuncs();
   else
     	mExtObject[Index]   = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::PartitionFunction);   
   mExtType[Index] = csObject;
  
   // Number of Partition Schemes
   Index++;
   if (mpActnPort->GetExtObjParScheme())
     	mExtObject[Index] = mpActnPort->GetObjNumExtParSchemes();
   else
     	mExtObject[Index]   = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::PartitionSchema);   
   mExtType[Index] = csObject; 

   // Number of Extended Objects
   Index++;
   if (mpActnPort->GetExtObjMethod())
     	mExtObject[Index] = mpActnPort->GetObjNumExtMethods();
   else
    	mExtObject[Index] = _T("0");
   csObject.Format(_T("%d"), OptimReportingContext::ExtendedObject);   
   mExtType[Index] = csObject;   
   
   for (int Cnt = 0; Cnt < ExtSize; Cnt++)
   {
      if (Cnt > 0)
      {
         reportingExtendedObjects->pNext = new OptimReportingNumberPair_t;
	      reportingExtendedObjects = reportingExtendedObjects->pNext;
      }      
      lValue = _ttol((const TCHAR *) mExtObject[Cnt]);
      reportingExtendedObjects->number1 = lValue; 
      lValue = _ttol((const TCHAR *) mExtType[Cnt]); 
      reportingExtendedObjects->number2 = lValue;
   }
   reportingExtendedObjects->pNext = NULL;
     
   return(TRUE);
}

/*---------------------------------------------------------------------------
 * OptimReportingContext:BuildImpactAnalysis
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::BuildImpactAnalysis()
{
   RtTraceEntry(OptimReportingContext::BuildImpactAnalysis);  
   long lTableDeletedRows = 0;
   long long lTableSaving = 0;
   long long lIndexSaving = 0;
   int iStepNum = 1;     
   OptimReportingTableImpAnalysis_t *reportingImpAnalysis;
   reportingImpAnalysis = new OptimReportingTableImpAnalysis_t;
   mpContext->pAnalysis = reportingImpAnalysis;
   OptimReportingIndexImpAnalysis_t *reportingIndexImpAnalysis;
   reportingIndexImpAnalysis = new OptimReportingIndexImpAnalysis_t;
   mpContext->pAnalysis->pIndexAnalysis = reportingIndexImpAnalysis;   
   mpContext->numImpactAnalysis = mpActnPort->GetStatIndxCount();
   //Create defaults
   wcscpy(reportingImpAnalysis->name, ORC_EMPTY_LSTRING);
   wcscpy(reportingImpAnalysis->type, ORC_EMPTY_LSTRING); 
   wcscpy(reportingImpAnalysis->status, ORC_EMPTY_LSTRING);
   reportingImpAnalysis->storageSavings = ORC_DEFAULT_VALUE;
  
   long DelStatStep = mpActnPort->GetStatIndxCount();   
   for ( int LocCnt = 1 ; LocCnt <=DelStatStep ; LocCnt++)
   {
      mpActnPort->PrimeStatIndx(LocCnt);
      if (LocCnt > 1)
      {
        reportingImpAnalysis->pNext = new OptimReportingTableImpAnalysis_t;
        reportingImpAnalysis = reportingImpAnalysis->pNext;
        wcscpy(reportingImpAnalysis->name, ORC_EMPTY_LSTRING);
        wcscpy(reportingImpAnalysis->type, ORC_EMPTY_LSTRING); 
        wcscpy(reportingImpAnalysis->status, ORC_EMPTY_LSTRING);
        reportingImpAnalysis->storageSavings = ORC_DEFAULT_VALUE;
      }     
      wcscpy(reportingImpAnalysis->name, mpActnPort->GetStatIndxTableName());
      wcscpy(reportingImpAnalysis->type, ORC_TABLE_STRING); 
      wcscpy(reportingImpAnalysis->status, ORC_EMPTY_LSTRING);   
      lTableDeletedRows = _ttol(mpActnPort->GetStatIndxDeletedRows());
      lTableSaving = lTableDeletedRows * _ttol(mpActnPort->GetStatIndxTblRowLen());
      //todo byte adjusting      
      if (lTableSaving > 0)
          reportingImpAnalysis->storageSavings = lTableSaving;
       else
          reportingImpAnalysis->storageSavings = ORC_DEFAULT_VALUE;
      
      int iNumTblIndxs = _ttoi(mpActnPort->GetStatIndxNumTblIndexes());
      reportingImpAnalysis->numIndexAnalysis = iNumTblIndxs;
      int iIndxCnt = 0;      
      while ( iIndxCnt < iNumTblIndxs ) 
      {
         wcscpy(reportingIndexImpAnalysis->name, ORC_EMPTY_LSTRING);
         wcscpy(reportingIndexImpAnalysis->type, ORC_EMPTY_LSTRING); 
         wcscpy(reportingIndexImpAnalysis->status, ORC_EMPTY_LSTRING);
         reportingIndexImpAnalysis->storageSavings = ORC_DEFAULT_VALUE;
         if (iIndxCnt > 0)
         {
           reportingIndexImpAnalysis->pNext = new OptimReportingIndexImpAnalysis_t;
           reportingIndexImpAnalysis = reportingIndexImpAnalysis->pNext;
           wcscpy(reportingIndexImpAnalysis->name, ORC_EMPTY_LSTRING);
           wcscpy(reportingIndexImpAnalysis->type, ORC_EMPTY_LSTRING); 
           wcscpy(reportingIndexImpAnalysis->status, ORC_EMPTY_LSTRING);
           reportingIndexImpAnalysis->storageSavings = ORC_DEFAULT_VALUE;
         }
         mpActnPort->PrimeStatIndx(iIndxCnt + iStepNum);
         wcscpy(reportingIndexImpAnalysis->name, mpActnPort->GetStatIndxIndexName());   
         wcscpy(reportingIndexImpAnalysis->type, ORC_INDEX_STRING);
         switch ( _ttoi(mpActnPort->GetStatIndxIndexStatus()))
         {
           case 0:
                wcscpy(reportingIndexImpAnalysis->status, RtString(RTDEFS_STR_NONE));
                break;
           case 1:                 
                wcscpy(reportingIndexImpAnalysis->status, RtString(RTDEFS_STR_PARTIAL));
                break;
           case 2:               
                wcscpy(reportingIndexImpAnalysis->status, RtString(RTDEFS_STR_FULL));
                break;
        }  
        
        if (lIndexSaving > 0)
            reportingIndexImpAnalysis->storageSavings = lIndexSaving;
         else
            reportingIndexImpAnalysis->storageSavings = ORC_DEFAULT_VALUE;      
        iIndxCnt++;
      }
      reportingIndexImpAnalysis->pNext = NULL;      
   }
   reportingImpAnalysis->pNext = NULL;

   return(TRUE);
}

/*---------------------------------------------------------------------------
 * OptimReportingContext:BuildJoinControl
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::BuildJoinControl()
{
   RtTraceEntry(OptimReportingContext::BuildJoinControl);

   CArray<RtTable *, RtTable *>    arrTables;
   RtTable   *pParent;
   int       iTable;
   CString   csWord1;   
   OptimReportingJoinControl_t *reportingJoinControl;
   reportingJoinControl = new OptimReportingJoinControl_t;
   mpContext->pJoinControl = reportingJoinControl;
   
   mpContext->numJoinControl = arrTables.GetSize();
   for (iTable=0; iTable<arrTables.GetSize(); iTable++)
   {
      if (iTable > 0)
      {
          reportingJoinControl->pNext = new OptimReportingJoinControl_t;
	       reportingJoinControl = reportingJoinControl->pNext;
      }
      wcscpy(reportingJoinControl->tableName, arrTables[iTable]->GetFullName());
      pParent = arrTables[iTable]->GetParent();  
      if (pParent == NULL)
      {          
          wcscpy(reportingJoinControl->joinedTable, RtString(RTACTION_STRING_STARTTABLE));
      }
      else
      {
         wcscpy(reportingJoinControl->tableName, pParent->GetFullName());
      }   
   }
   reportingJoinControl->pNext = NULL;

   return(TRUE);
}

/*---------------------------------------------------------------------------
 * OptimReportingContext:BuildCommonErrorMessages
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::BuildCommonErrorMessages()
{
   RtTraceEntry(OptimReportingContext::BuildCommonErrorMessages);
   
   mpContext->numMessages = mpActnPort->GetErrorMsgCount() +
                            mpActnPort->GetWarnMsgCount()  +
                            mpActnPort->GetInfoMsgCount()  +
                            mpActnPort->GetExitMsgCount();
   
   if (mpActnPort->GetErrorMsgCount() > 0)
   {
      OptimReportingMessage_t *reportingMessage;
      reportingMessage = new OptimReportingMessage_t;
      mpContext->pMessages = reportingMessage;

      for ( int i = 1 ; i <= mpActnPort->GetErrorMsgCount() ; i++)
	   { 
         if (i > 1)
         {
	        reportingMessage->pNext = new OptimReportingMessage_t;
	        reportingMessage = reportingMessage->pNext;
         }
         mpActnPort->PrimeMsg(ActnPort::TypeError,i);           
         reportingMessage->id = i;      
         wcscpy(reportingMessage->message,mpActnPort->GetCurMsg());
         reportingMessage->type =  OptimReportingContext::Error;	 
      }
	   reportingMessage->pNext = NULL;
   }

   return(TRUE);
}

/*---------------------------------------------------------------------------
 * OptimReportingContext:BuildCommonWarnMessages
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::BuildCommonWarnMessages()
{
   RtTraceEntry(OptimReportingContext::BuildCommonWarnMessages);
   
   if (mpActnPort->GetWarnMsgCount() > 0)
   {
     OptimReportingMessage_t *reportingMessage;
	  reportingMessage = new OptimReportingMessage_t;      
     mpContext->pMessages = reportingMessage;    
                
     for ( int i = 1 ; i <= mpActnPort->GetWarnMsgCount() ; i++)
	  { 
		   if (i > 1)
		   {
		      reportingMessage->pNext = new OptimReportingMessage_t;            
		      reportingMessage = reportingMessage->pNext;               
		   }
         mpActnPort->PrimeMsg(ActnPort::TypeWarning,i);          
         reportingMessage->id = i;       
         wcscpy(reportingMessage->message,mpActnPort->GetCurMsg());        
         reportingMessage->type = OptimReportingContext::Warning;		 
     }
	  reportingMessage->pNext = NULL;
   }

   return(TRUE); 
}


/*---------------------------------------------------------------------------
 * OptimReportingContext:BuildCommonInfoMessages
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::BuildCommonInfoMessages()
{
   RtTraceEntry(OptimReportingContext::BuildCommonInfoMessages);
  
   if (mpActnPort->GetInfoMsgCount() > 0)
   {
      OptimReportingMessage_t *reportingMessage;
      reportingMessage = new OptimReportingMessage_t;
      mpContext->pMessages = reportingMessage;

      for ( int i = 1 ; i <= mpActnPort->GetInfoMsgCount() ; i++)
	   { 
         if (i > 1)
         {
	        reportingMessage->pNext = new OptimReportingMessage_t;
	        reportingMessage = reportingMessage->pNext;
         }
         mpActnPort->PrimeMsg(ActnPort::TypeInfo,i);         
         reportingMessage->id = i;       
         wcscpy(reportingMessage->message,mpActnPort->GetCurMsg());  
         reportingMessage->type = OptimReportingContext::Info;		 
      }
	   reportingMessage->pNext = NULL;
   }

   return(TRUE); 
}

/*---------------------------------------------------------------------------
 * OptimReportingContext:BuildCommonExitMessages
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::BuildCommonExitMessages()
{
   RtTraceEntry(OptimReportingContext::BuildCommonExitMessages);
  
   if (mpActnPort->GetExitMsgCount() > 0)
   {
      OptimReportingMessage_t *reportingMessage;
      reportingMessage = new OptimReportingMessage_t;
      mpContext->pMessages = reportingMessage;

      for ( int i = 1 ; i <= mpActnPort->GetExitMsgCount() ; i++)
	   { 
         if (i > 1)
         {
	        reportingMessage->pNext = new OptimReportingMessage_t;
	        reportingMessage = reportingMessage->pNext;
         }
         mpActnPort->PrimeMsg(ActnPort::TypeExit,i);          
         reportingMessage->id =i; 
         wcscpy(reportingMessage->message,mpActnPort->GetCurMsg());         
         reportingMessage->type = OptimReportingContext::Exit;	 
      }
	   reportingMessage->pNext = NULL;
   }

   return(TRUE); 
}   

/*---------------------------------------------------------------------------
 * OptimReportingContext:BuildRequestOptions
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::BuildRequestOptions()
{
   RtTraceEntry(OptimReportingContext::BuildRequestOptions);
     
   OptimReportingNameValuePair_t *reportingRequestOption;
   reportingRequestOption = new OptimReportingNameValuePair_t;
   mpContext->pRequestOptions = reportingRequestOption;

   mpContext->numRequestOptions = ORC_DEFAULT_VALUE;  
   wcscpy(reportingRequestOption->name, ORC_EXTRACT_BOTH);
   wcscpy(reportingRequestOption->value, _T("2")); 
   
   //todo
   switch (mActionType)
   {
      case  ExtractAction:
      case  ArchiveAction:
      case  RestoreAction: 
              mpContext->numRequestOptions = 1;
             switch (mpActnPort->GetExtOpt())
             {
               case DsxEX::Data:                   
                 wcscpy(reportingRequestOption->name, ORC_EXTRACT_DATA);
                 wcscpy(reportingRequestOption->value, _T("0"));                                      
               break;
               case DsxEX::Objects:               
                  wcscpy(reportingRequestOption->name, ORC_EXTRACT_OBJECTS);
                  wcscpy(reportingRequestOption->value, _T("1"));                                   
               break;
               case DsxEX::Both:
                   wcscpy(reportingRequestOption->name, ORC_EXTRACT_BOTH);
                   wcscpy(reportingRequestOption->value, _T("2"));                                    
               break;
            }            
      break;
      
      case  ConvertAction:
      case  DeleteAction:
      case  LoadAction:
      case  InsertAction:
      case  Import:
      case  ReportAction:           
            wcscpy(reportingRequestOption->value, ORC_EMPTY_LSTRING);           
            wcscpy(reportingRequestOption->name, ORC_EMPTY_LSTRING);
      break;    

      default:
      break;

   }   
   reportingRequestOption->pNext = NULL;

   return(TRUE); 
}

/*---------------------------------------------------------------------------
 * OptimReportingContext:BuildRelationship
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::BuildRelationship()
{
   RtTraceEntry(OptimReportingContext::BuildRelationship);

   int ienum=0;
   int iValue;
   long StatRelUsgSteps = mpActnPort->GetStatRelUsgCount();
   CString csParAccess = _T("");
   CString csChildAccess = _T("");
   CString csParent = _T("");
   CString csChild = _T("");
   CString csRel = _T("");
   CString csParKey = _T("");
   CString csChildKey = _T("");
   CString csAccessMsg = _T("");  
   OptimReportingRelationship_t *reportingRelationship;
   reportingRelationship = new OptimReportingRelationship_t;
   mpContext->pRelationships = reportingRelationship;
   mpContext->numRelationships = StatRelUsgSteps;

   // Table Access Control    
   for (int RelCnt = 1 ; RelCnt <= StatRelUsgSteps ;RelCnt++)   
   {
       mpActnPort->PrimeStatRelUsg(RelCnt);       
       if (RelCnt > 1)
       {
           reportingRelationship->pNext = new OptimReportingRelationship_t;
           reportingRelationship = reportingRelationship->pNext;
       }
       csParKey = RtString(REPT_ROWDETAILS_DATANOTSHOWN);
       csChildKey =  RtString(REPT_ROWDETAILS_DATANOTSHOWN);
       csAccessMsg = _T("");
       // Retrieve the table name        
       wcscpy(reportingRelationship->parentName, mpActnPort->GetStatRelUsgParentTable());
       wcscpy(reportingRelationship->childName, mpActnPort->GetStatRelUsgChildTable());        
       wcscpy(reportingRelationship->relationshipName, mpActnPort->GetStatRelUsgRelationshipName());
   
      // Below lines not required if Netezza Fast Migration enabled
      if ( mpActnPort->GetNetezzaFM() == FALSE)
      {
         switch  (_ttoi(mpActnPort->GetStatRelUsgParentAccess())) {
           case DXX_STRATEGY_NO_KEYS:
               csParAccess = RtString(RTACTION_ACCESS_NONE);
               ienum = OptimReportingContext::ACCESS_NONE;
               break;              
          case DXX_STRATEGY_FORCE_SCAN:
               csParAccess = RtString(  RTACTION_ACCESS_FORCESCAN);
               csAccessMsg = RtString(  RTACTION_XTRCFORCEMSG);
               ienum = OptimReportingContext::RELFORCE_SCAN;
               break;
          case DXX_STRATEGY_FORCE_SCAN_SUBQUERY:
               csParAccess = RtString(  RTACTION_ACCESS_FORCESCANSUBQUERY);
               csAccessMsg = RtString(  RTACTION_XTRCFORCEMSG);
               ienum = OptimReportingContext::FORCE_SCANSUBQUERY;
               break;
          case DXX_STRATEGY_FORCE_LOOKUP:
               csParAccess = RtString( RTACTION_ACCESS_FORCEKEY);
               csAccessMsg = RtString(  RTACTION_XTRCFORCEMSG);
               csParKey = mpActnPort->GetStatRelUsgParKeyLimit();
               ienum = OptimReportingContext::FORCE_KEY;
               break;
          case DXX_STRATEGY_ONLY_1_KEY:
          case DXX_STRATEGY_LOOKUP:
          case DXX_STRATEGY_LOOKUP_TOO_FEW_KEYS:
               csParAccess = RtString(RTACTION_ACCESS_KEY); 
               csParKey = mpActnPort->GetStatRelUsgParKeyLimit();
               ienum = OptimReportingContext::ACCESS_KEY;
               break;              
         case DXX_STRATEGY_SCAN_TOO_MANY_KEYS:
         case DXX_STRATEGY_SCAN_NO_INDEX:
               csParAccess = RtString( RTACTION_ACCESS_SCAN);
               ienum = OptimReportingContext::ACCESS_SCAN;
               break;             
         case DXX_STRATEGY_SCAN_SUBQUERY:
               csParAccess = RtString(RTACTION_ACCESS_SCANSUBQUERY);
               ienum = OptimReportingContext::ACCESS_SCANSUBQUERY;
               break;
         default:
               csParAccess = RtString(RTXF_STRING_NAME_UNAVAILABLE);
               ienum = ORC_DEFAULT_VALUE;
               break;
   }    
   reportingRelationship->parentAccessType = ienum;
   switch  (_ttoi(mpActnPort->GetStatRelUsgChildAccess())) {
           case DXX_STRATEGY_NO_KEYS:
               csChildAccess = RtString(RTACTION_ACCESS_NONE);
               ienum = OptimReportingContext::ACCESS_NONE;
               break;              
          case DXX_STRATEGY_FORCE_SCAN:
               csChildAccess = RtString( RTACTION_ACCESS_FORCESCAN);
               csAccessMsg = RtString(  RTACTION_XTRCFORCEMSG);
               ienum = OptimReportingContext::RELFORCE_SCAN;
               break;
          case DXX_STRATEGY_FORCE_SCAN_SUBQUERY:
               csChildAccess = RtString( RTACTION_ACCESS_FORCESCANSUBQUERY);
               csAccessMsg = RtString(  RTACTION_XTRCFORCEMSG);
               ienum = OptimReportingContext::FORCE_SCANSUBQUERY;
               break;
          case DXX_STRATEGY_FORCE_LOOKUP:
               csChildAccess = RtString(RTACTION_ACCESS_FORCEKEY);
               csAccessMsg = RtString(  RTACTION_XTRCFORCEMSG);
               csChildKey = mpActnPort->GetStatRelUsgChildKeyLimit();
               ienum = OptimReportingContext::FORCE_KEY;
               break;
          case DXX_STRATEGY_ONLY_1_KEY:
          case DXX_STRATEGY_LOOKUP:
          case DXX_STRATEGY_LOOKUP_TOO_FEW_KEYS:
               csChildAccess = RtString(RTACTION_ACCESS_KEY);
               csChildKey = mpActnPort->GetStatRelUsgChildKeyLimit();
               ienum = OptimReportingContext::ACCESS_KEY;
               break;              
         case DXX_STRATEGY_SCAN_TOO_MANY_KEYS:
         case DXX_STRATEGY_SCAN_NO_INDEX:
               csChildAccess = RtString( RTACTION_ACCESS_SCAN);
               ienum = OptimReportingContext::ACCESS_SCAN;
               break;             
         case DXX_STRATEGY_SCAN_SUBQUERY:
               csChildAccess = RtString(RTACTION_ACCESS_SCANSUBQUERY);
               ienum = OptimReportingContext::ACCESS_SCANSUBQUERY;
               break;
         default:
               csChildAccess = RtString(RTXF_STRING_NAME_UNAVAILABLE);
               ienum = ORC_DEFAULT_VALUE;
               break;
      }      
      reportingRelationship->childAccessType = ienum;      
      iValue = _ttoi(csParKey);
      reportingRelationship->parentKeyLimit = iValue;
      iValue = _ttoi(csChildKey); 
      reportingRelationship->childKeyLimit = iValue;
     }  
   }
   reportingRelationship->pNext = NULL;

   return(TRUE);   
}

/*---------------------------------------------------------------------------
 * OptimReportingContext:BuildStatistics()
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::BuildStatistics()
{
   RtTraceEntry(OptimReportingContext::BuildStatistics);

   CString csWord, csWord2;  
   CString csPercent;
   CString csExtr = "";   
   CString csPercentageFormat = _T("%.1d%%%%");      
   int ienum = 0;
   int iValue = 0;
   long long lValue = 0;   
   OptimReportingStatistics_t *reportingStatistics;
   reportingStatistics = new OptimReportingStatistics_t;
   mpContext->pStatistics = reportingStatistics;
   mpContext->numStatistics = mpActnPort->GetStatStepCount();
   
   // create defaults
   reportingStatistics->estimatedRowCount = ORC_DEFAULT_VALUE;
   reportingStatistics->fetchBuffer = ORC_DEFAULT_VALUE;
   reportingStatistics->foreignKeyIndex = ORC_DEFAULT_VALUE;
   reportingStatistics->foreignKeyNoIndex = ORC_DEFAULT_VALUE;
   reportingStatistics->primaryKeyIndex = ORC_DEFAULT_VALUE;
   reportingStatistics->primaryKeyNoIndex = ORC_DEFAULT_VALUE;
   reportingStatistics->dbDataPerSecond = ORC_DEFAULT_VALUE;
   reportingStatistics->dbProcessTime = ORC_DEFAULT_VALUE;
   reportingStatistics->dbRowsFetched = ORC_DEFAULT_VALUE;
   reportingStatistics->dbRowsPerSecond = ORC_DEFAULT_VALUE;
   reportingStatistics->dbRowsWritten = ORC_DEFAULT_VALUE;
   reportingStatistics->dbTimeInAct = ORC_DEFAULT_VALUE;
   reportingStatistics->dbTimeInDB = ORC_DEFAULT_VALUE;
   wcscpy(reportingStatistics->name, ORC_EMPTY_LSTRING);
   wcscpy(reportingStatistics->dbmsVendor, ORC_EMPTY_LSTRING);
   wcscpy(reportingStatistics->dbmsVersion, ORC_EMPTY_LSTRING);
   wcscpy(reportingStatistics->relName, ORC_EMPTY_LSTRING);
   wcscpy(reportingStatistics->relSql, ORC_EMPTY_LSTRING);
   reportingStatistics->childStrategy = ORC_DEFAULT_VALUE;
   reportingStatistics->columnCount = ORC_DEFAULT_VALUE;
   reportingStatistics->databaseConnections = ORC_DEFAULT_VALUE;
   reportingStatistics->method = ORC_DEFAULT_VALUE;
   reportingStatistics->parentStrategy = ORC_DEFAULT_VALUE;
   reportingStatistics->rowLength = ORC_DEFAULT_VALUE;
   reportingStatistics->stepNumber = ORC_DEFAULT_VALUE;
   wcscpy(reportingStatistics->connectionMode, ORC_EMPTY_LSTRING);
   wcscpy(reportingStatistics->whereClause, ORC_EMPTY_LSTRING);
   wcscpy(reportingStatistics->rowid, ORC_EMPTY_LSTRING);  
   reportingStatistics->relAccess = ORC_DEFAULT_VALUE;
   reportingStatistics->relDb2Cost = ORC_DEFAULT_VALUE;
   reportingStatistics->relDb2ScanCost = ORC_DEFAULT_VALUE;
   reportingStatistics->relDirection = ORC_DEFAULT_VALUE;
   reportingStatistics->relKeyLength = ORC_DEFAULT_VALUE;
   reportingStatistics->relKeyPerCur = ORC_DEFAULT_VALUE;
   reportingStatistics->relLookupKey = ORC_DEFAULT_VALUE;
   reportingStatistics->relType = ORC_DEFAULT_VALUE;  
   reportingStatistics->dbAccessType = ORC_DEFAULT_VALUE;
   reportingStatistics->dbKeysPer = ORC_DEFAULT_VALUE;
   reportingStatistics->dbOpenCursor = ORC_DEFAULT_VALUE;
   reportingStatistics->cycle = FALSE; 
   reportingStatistics->lobs= FALSE;
   reportingStatistics->selectUniqueRecord= FALSE;
   reportingStatistics->relIndex= FALSE;
   wcscpy(reportingStatistics->compTableName, ORC_EMPTY_LSTRING);
   wcscpy(reportingStatistics->compRatio, ORC_EMPTY_LSTRING);
   
   //  Build the Extract/Archive/Delete Statistical Report   
   long StatSteps = mpActnPort->GetStatStepCount();
   for ( int LocCnt = 1 ; LocCnt <=StatSteps ; LocCnt++)
   {
       mpActnPort->PrimeStatStep(LocCnt);        
       if (LocCnt > 1)
       {
	        reportingStatistics->pNext = new OptimReportingStatistics_t;
	        reportingStatistics = reportingStatistics->pNext;
           // Create Defaults
           reportingStatistics->estimatedRowCount = ORC_DEFAULT_VALUE;
           reportingStatistics->fetchBuffer = ORC_DEFAULT_VALUE;
           reportingStatistics->foreignKeyIndex = ORC_DEFAULT_VALUE;
           reportingStatistics->foreignKeyNoIndex = ORC_DEFAULT_VALUE;
           reportingStatistics->primaryKeyIndex = ORC_DEFAULT_VALUE;
           reportingStatistics->primaryKeyNoIndex = ORC_DEFAULT_VALUE;
           reportingStatistics->dbDataPerSecond = ORC_DEFAULT_VALUE;
           reportingStatistics->dbProcessTime = ORC_DEFAULT_VALUE;
           reportingStatistics->dbRowsFetched = ORC_DEFAULT_VALUE;
           reportingStatistics->dbRowsPerSecond = ORC_DEFAULT_VALUE;
           reportingStatistics->dbRowsWritten = ORC_DEFAULT_VALUE;
           reportingStatistics->dbTimeInAct = ORC_DEFAULT_VALUE;
           reportingStatistics->dbTimeInDB = ORC_DEFAULT_VALUE;
           wcscpy(reportingStatistics->name, ORC_EMPTY_LSTRING);
           wcscpy(reportingStatistics->dbmsVendor, ORC_EMPTY_LSTRING);
           wcscpy(reportingStatistics->dbmsVersion, ORC_EMPTY_LSTRING);
           wcscpy(reportingStatistics->relName, ORC_EMPTY_LSTRING);
           wcscpy(reportingStatistics->relSql, ORC_EMPTY_LSTRING);
           reportingStatistics->childStrategy = ORC_DEFAULT_VALUE;
           reportingStatistics->columnCount = ORC_DEFAULT_VALUE;
           reportingStatistics->databaseConnections = ORC_DEFAULT_VALUE;
           reportingStatistics->method = ORC_DEFAULT_VALUE;
           reportingStatistics->parentStrategy = ORC_DEFAULT_VALUE;
           reportingStatistics->rowLength = ORC_DEFAULT_VALUE;
           reportingStatistics->stepNumber = ORC_DEFAULT_VALUE;
           wcscpy(reportingStatistics->connectionMode, ORC_EMPTY_LSTRING);
           wcscpy(reportingStatistics->whereClause, ORC_EMPTY_LSTRING);
           wcscpy(reportingStatistics->rowid, ORC_EMPTY_LSTRING);  
           reportingStatistics->relAccess = ORC_DEFAULT_VALUE;
           reportingStatistics->relDb2Cost = ORC_DEFAULT_VALUE;
           reportingStatistics->relDb2ScanCost = ORC_DEFAULT_VALUE;
           reportingStatistics->relDirection = ORC_DEFAULT_VALUE;
           reportingStatistics->relKeyLength = ORC_DEFAULT_VALUE;
           reportingStatistics->relKeyPerCur = ORC_DEFAULT_VALUE;
           reportingStatistics->relLookupKey = ORC_DEFAULT_VALUE;
           reportingStatistics->relType = ORC_DEFAULT_VALUE;           
           reportingStatistics->dbAccessType = ORC_DEFAULT_VALUE;
           reportingStatistics->dbKeysPer = ORC_DEFAULT_VALUE;
           reportingStatistics->dbOpenCursor = ORC_DEFAULT_VALUE;
           reportingStatistics->cycle = FALSE; 
           reportingStatistics->lobs= FALSE;
           reportingStatistics->selectUniqueRecord= FALSE;
           reportingStatistics->relIndex= FALSE;
           wcscpy(reportingStatistics->compTableName, ORC_EMPTY_LSTRING);
           wcscpy(reportingStatistics->compRatio, ORC_EMPTY_LSTRING);
       }
       RtRDBA  dba;      
       UCHAR Type = _ttoi(mpActnPort->GetStatStepDBMSType());
       csWord  = dba.GetDBTypeLiteral(Type);
       mDBMSType = csWord;
       mDBMSVer = mpActnPort->GetStatStepDBMSVerStr();
       wcscpy(reportingStatistics->dbmsVendor, mDBMSType);
	    wcscpy(reportingStatistics->dbmsVersion, mDBMSVer); ;
       mCompression = streq(mpActnPort->GetStatStepFileComp(),AC_TRUE)
                        ? (const TCHAR *) RtString(RTOVER_STR_YES)
                        : (const TCHAR *) RtString(RTOVER_STR_NO);   
      iValue = _ttoi(mpActnPort->GetStatStepStepNum());
      reportingStatistics->stepNumber = iValue;  
      CString csTableName;
      csTableName.Format(_T("%s.%s.%s"),mpActnPort->GetStatStepDBAliasName(),
                               mpActnPort->GetStatStepCreatorID(),
                               mpActnPort->GetStatStepTableName());      
      wcscpy(reportingStatistics->name, csTableName);            
      iValue = _ttoi(mpActnPort->GetStatStepNumTblCols());
      reportingStatistics->columnCount = iValue;
      csWord2 = streq(mpActnPort->GetStatStepInACycle(),AC_TRUE)
                                                  ? (const TCHAR *) RtString(RTOVER_STR_YES)
                                                  : (const TCHAR *) RtString(RTOVER_STR_NO);
      reportingStatistics->cycle = FALSE;
      if (csWord2 == RtString(RTOVER_STR_YES))
         reportingStatistics->cycle = TRUE;
      csWord2 = streq(mpActnPort->GetStatStepTableHasLOBs(),AC_TRUE)
                                                  ? (const TCHAR *) RtString(RTOVER_STR_YES)
                                                  : (const TCHAR *) RtString(RTOVER_STR_NO);
      reportingStatistics->lobs = FALSE;
      if (csWord2 == RtString(RTOVER_STR_YES))
         reportingStatistics->lobs = TRUE;
      if (!(streq(mpActnPort->GetStatStepNumTblRows(),AC_NA) || Type == RDB_NETEZZA))
      {
        lValue = _ttol((const TCHAR *)mpActnPort->GetStatStepNumTblRows());
        reportingStatistics->estimatedRowCount = lValue;
      }
      else
        reportingStatistics->estimatedRowCount =  ORC_DEFAULT_VALUE;   

      iValue = _ttoi(mpActnPort->GetStatStepRowLen());
      reportingStatistics->rowLength = iValue;
      iValue = _ttoi(mpActnPort->GetStatStepNumThreads());
      reportingStatistics->databaseConnections = iValue;     

      // Add Select With UR.        
      csWord2 = streq(mpActnPort->GetStatStepSelectWithUR(), AC_TRUE)
                   ? (const TCHAR *) RtString(RTOVER_STR_YES)
                   : (const TCHAR *) RtString(RTOVER_STR_NO);
      reportingStatistics->selectUniqueRecord = FALSE;
      if (csWord2 == RtString(RTOVER_STR_YES))
         reportingStatistics->selectUniqueRecord = TRUE;
      lValue = _ttol((const TCHAR *)mpActnPort->GetStatStepExtractMaxAllocation());
      reportingStatistics->fetchBuffer = lValue;
      if(Type == RDB_TERADATA)
      {
         csWord.Empty();
         DWORD ConnMode =  _ttoi(mpActnPort->GetStatStepConnectionMode());
         if(ConnMode == DX_TERA_CONN_MODE_TPT)
            mConnMode = RtString(RTCONS_TERA_CONN_MODE_TPT);
         else if(ConnMode == DX_TERA_CONN_MODE_CLI)
            mConnMode = RtString(RTCONS_TERA_CONN_MODE_CLI);
         wcscpy(reportingStatistics->connectionMode, mConnMode);
      }
      else
      {
         wcscpy(reportingStatistics->connectionMode, ORC_EMPTY_LSTRING);
      }

     if (streq(mpActnPort->GetStatStepStrategyInfoAvail(),AC_TRUE)) 
     {
        lValue = _ttol((const TCHAR *)mpActnPort->GetStatStepNumPKsWithIndex());
        reportingStatistics->primaryKeyIndex = lValue;
        lValue = _ttol((const TCHAR *)mpActnPort->GetStatStepNumPKsWoIndex());
        reportingStatistics->primaryKeyNoIndex = lValue;
        lValue = _ttol((const TCHAR *)mpActnPort->GetStatStepNumFKsWithIndex());
        reportingStatistics->foreignKeyIndex  = lValue;
        lValue = _ttol((const TCHAR *)mpActnPort->GetStatStepNumFKsWoIndex());
        reportingStatistics->foreignKeyNoIndex = lValue;
        switch  (_ttoi(mpActnPort->GetStatStepParStrategy())) {
          case DXX_STRATEGY_NO_KEYS:
               csWord = RtString(RTACTION_STRATEGY_NO_KEYS);
               ienum = OptimReportingContext::NO_KEYS;
               break;
          case DXX_STRATEGY_ONLY_1_KEY:
               csWord = RtString(RTACTION_STRATEGY_ONLY_1_KEY);
               ienum = OptimReportingContext::ONLY_1_KEY;
               break;
          case DXX_STRATEGY_FORCE_SCAN:
               csWord = RtString( RTACTION_STRATEGY_FORCE_SCAN);
               ienum = OptimReportingContext::FORCE_SCAN ;
               break;
          case DXX_STRATEGY_FORCE_SCAN_SUBQUERY:
               csWord = RtString(RTACTION_STRATEGY_FORCE_SCAN_SUBQUERY);
               ienum = OptimReportingContext::FORCE_LOOKUP;
               break;
          case DXX_STRATEGY_FORCE_LOOKUP:
               csWord = RtString(RTACTION_STRATEGY_FORCE_LOOKUP);
               ienum = OptimReportingContext::LOOKUP;
               break;
          case DXX_STRATEGY_LOOKUP:
               csWord = RtString(RTACTION_STRATEGY_LOOKUP);
               ienum = OptimReportingContext::LOOKUP_TOO_FEW_KEYS;
               break;
          case DXX_STRATEGY_SCAN_NO_INDEX:
               csWord = RtString(RTACTION_STRATEGY_SCAN_NO_INDEX);
               ienum = OptimReportingContext::SCAN_NO_INDEX;
               break;
          case DXX_STRATEGY_SCAN_TOO_MANY_KEYS:
               csWord = RtString(RTACTION_STRATEGY_SCAN_TOO_MANY_KEYS);
               ienum = OptimReportingContext::SCAN_TOO_MANY_KEYS;
               break;
          case DXX_STRATEGY_SCAN_SUBQUERY:
               csWord = RtString(RTACTION_STRATEGY_SCAN_SUBQUERY);
               ienum = OptimReportingContext::SCAN_SUBQUERY;
               break;
          case DXX_STRATEGY_LOOKUP_TOO_FEW_KEYS:
               csWord = RtString(RTACTION_STRATEGY_LOOKUP_TOO_FEW_KEYS);
               ienum = OptimReportingContext::FORCE_SCAN_SUBQUERY;
               break;
          default:
               csWord = RtString(RTXF_STRING_NAME_UNKNOWN);
               ienum = ORC_DEFAULT_VALUE;
               break;
        }        
        reportingStatistics->parentStrategy = ienum;
        
        switch  (_ttoi(mpActnPort->GetStatStepDepStrategy())) {
         case DXX_STRATEGY_NO_KEYS:
               csWord = RtString(RTACTION_STRATEGY_NO_KEYS);
               ienum = OptimReportingContext::NO_KEYS;
               break;
          case DXX_STRATEGY_ONLY_1_KEY:
               csWord = RtString(RTACTION_STRATEGY_ONLY_1_KEY);
               ienum = OptimReportingContext::ONLY_1_KEY;
               break;
          case DXX_STRATEGY_FORCE_SCAN:
               csWord = RtString( RTACTION_STRATEGY_FORCE_SCAN);
               ienum = OptimReportingContext::FORCE_SCAN ;
               break;
          case DXX_STRATEGY_FORCE_SCAN_SUBQUERY:
               csWord = RtString(RTACTION_STRATEGY_FORCE_SCAN_SUBQUERY);
               ienum = OptimReportingContext::FORCE_LOOKUP;
               break;
          case DXX_STRATEGY_FORCE_LOOKUP:
               csWord = RtString(RTACTION_STRATEGY_FORCE_LOOKUP);
               ienum = OptimReportingContext::LOOKUP;
               break;
          case DXX_STRATEGY_LOOKUP:
               csWord = RtString(RTACTION_STRATEGY_LOOKUP);
               ienum = OptimReportingContext::LOOKUP_TOO_FEW_KEYS;
               break;
          case DXX_STRATEGY_SCAN_NO_INDEX:
               csWord = RtString(RTACTION_STRATEGY_SCAN_NO_INDEX);
               ienum = OptimReportingContext::SCAN_NO_INDEX;
               break;
          case DXX_STRATEGY_SCAN_TOO_MANY_KEYS:
               csWord = RtString(RTACTION_STRATEGY_SCAN_TOO_MANY_KEYS);
               ienum = OptimReportingContext::SCAN_TOO_MANY_KEYS;
               break;
          case DXX_STRATEGY_SCAN_SUBQUERY:
               csWord = RtString(RTACTION_STRATEGY_SCAN_SUBQUERY);
               ienum = OptimReportingContext::SCAN_SUBQUERY;
               break;
          case DXX_STRATEGY_LOOKUP_TOO_FEW_KEYS:
               csWord = RtString(RTACTION_STRATEGY_LOOKUP_TOO_FEW_KEYS);
               ienum = OptimReportingContext::FORCE_SCAN_SUBQUERY;
               break;
          default:
               csWord = RtString(RTXF_STRING_NAME_UNKNOWN);
               ienum = ORC_DEFAULT_VALUE;
               break;
        }
        reportingStatistics->childStrategy = ienum;
      
      }
      else {     
        reportingStatistics->primaryKeyIndex = ORC_DEFAULT_VALUE;           
        reportingStatistics->primaryKeyNoIndex = ORC_DEFAULT_VALUE;        
        reportingStatistics->foreignKeyIndex = ORC_DEFAULT_VALUE;           
        reportingStatistics->foreignKeyNoIndex = ORC_DEFAULT_VALUE;         
        reportingStatistics->parentStrategy = ORC_DEFAULT_VALUE;             
        reportingStatistics->childStrategy = ORC_DEFAULT_VALUE; 
      }      
      wcscpy(reportingStatistics->whereClause, mpActnPort->GetStatStepUserWhereClause());         
               
      if (mpActnPort->GetCurTblDMLOprByRowID())
      {
        mRowId = RtString(EXTR_RSDL_EXTROPTROWID);
	     wcscpy(reportingStatistics->rowid, mRowId);  		 
      }
      else
      {
        wcscpy(reportingStatistics->rowid, ORC_EMPTY_LSTRING);
      }
      

      /*-----------------------------------------------------------*/
      /*  Now process the Relationship for each step               */
      /*-----------------------------------------------------------*/     
      long StatRel = mpActnPort->GetStatRelCount();
      if (StatRel > 0) 
      {
         for (int LocCnt = 1 ; LocCnt <= StatRel ; LocCnt++) 
         {
            mpActnPort->PrimeStatRel(LocCnt);            
            _TUCHAR RelDirection = _ttoi(mpActnPort->GetStatRelDirection());
            if (RelDirection != _T('P')) 
            {
               mRelName.Format(_T("%s.%s.%s.%s"),mpActnPort->GetStatRelRelDBAliasName(),
                                           mpActnPort->GetStatRelRelCreatorID(),
                                           mpActnPort->GetStatRelRelTableName(),
                                           mpActnPort->GetStatRelRelName());
               wcscpy(reportingStatistics->relName, mRelName);
               reportingStatistics->relDirection = OptimReportingContext::Dependent;            
            }  // endif (RelDirection != _T('P')) 
            else
            {
              mRelName = RtString(RTKEY_PRIKEY_STR);
              wcscpy(reportingStatistics->relName, mRelName);
              reportingStatistics->relDirection = OptimReportingContext::Parent;

            }        
            iValue = _ttoi(mpActnPort->GetStatRelNumKeysToLookup());
            reportingStatistics->relLookupKey = iValue;
            switch (RelDirection) {
              case _T('P'):
                csWord = RtString(RTRL_STRING_PARENT);
                ienum = OptimReportingContext::Parent;
                break;
              case _T('D'):
                csWord = RtString(RTRL_STRING_DEPENDENT);
                ienum = OptimReportingContext::Dependent;
                break;
              default:
                csWord = RtString(RTXF_STRING_NAME_UNKNOWN);
                ienum = ORC_DEFAULT_VALUE;
                break;
            }
            reportingStatistics->relType = ienum;                        
            csWord2 = streq(mpActnPort->GetStatRelIndexed(),AC_TRUE)
                                                    ? (const TCHAR *) RtString(RTOVER_STR_YES)
                                                    : (const TCHAR *) RtString(RTOVER_STR_NO);  
            reportingStatistics->relIndex = FALSE;
            if (csWord2 == RtString(RTOVER_STR_YES))
               reportingStatistics->relIndex = TRUE;
            iValue = _ttoi(mpActnPort->GetStatRelCompoundKeyCount());
            reportingStatistics->relKeyPerCur = iValue;            
            if (Type == RDB_DB2CS || Type == RDB_DB2MVS) {              
              iValue = _ttoi(mpActnPort->GetStatRelDB2KeyLookupCost());
              reportingStatistics->relDb2Cost = iValue;
              iValue = _ttoi(mpActnPort->GetStatRelDB2ScanCost());
              reportingStatistics->relDb2ScanCost = iValue;
            }
            else {              
              reportingStatistics->relDb2Cost = ORC_DEFAULT_VALUE;
              reportingStatistics->relDb2ScanCost = ORC_DEFAULT_VALUE;
            }
            iValue = _ttoi(mpActnPort->GetStatRelKeyLength());
            reportingStatistics->relKeyLength = iValue;         
            switch  (_ttoi(mpActnPort->GetStatRelForcedAccess())) {
              case DX_FORCE_NONE:
                   csWord = RtString(RTACTION_STRATEGY_NONE);
                   ienum = OptimReportingContext::FORCE_NONE;
                   break;
              case DX_FORCE_SCAN:
                   csWord = RtString(RTACTION_STRATEGY_FORCE_SCAN);
                   ienum = OptimReportingContext::FORCE_NONE;
                   break;
              case DX_FORCE_KEYLOOKUP:
                   csWord = RtString(RTACTION_STRATEGY_FORCE_LOOKUP);
                   ienum = OptimReportingContext::FORCE_NONE;
                   break;
              default:
                csWord = RtString(RTXF_STRING_NAME_UNKNOWN);
                ienum = ORC_DEFAULT_VALUE;
                break;
            }              
            reportingStatistics->relAccess = ienum;        
            wcscpy(reportingStatistics->relSql, mpActnPort->GetStatRelKeyLookupSQL() );              
         }
      }
      else
      {
         wcscpy(reportingStatistics->relName, ORC_EMPTY_LSTRING);
         wcscpy(reportingStatistics->relSql, ORC_EMPTY_LSTRING);
         reportingStatistics->relAccess  = ORC_DEFAULT_VALUE;
	      reportingStatistics->relDb2Cost = ORC_DEFAULT_VALUE;
	      reportingStatistics->relDb2ScanCost = ORC_DEFAULT_VALUE;
	      reportingStatistics->relDirection = ORC_DEFAULT_VALUE;
	      reportingStatistics->relKeyLength = ORC_DEFAULT_VALUE;
	      reportingStatistics->relKeyPerCur = ORC_DEFAULT_VALUE;
	      reportingStatistics->relLookupKey = ORC_DEFAULT_VALUE;
	      reportingStatistics->relType = ORC_DEFAULT_VALUE;         
      }
       

      /*-----------------------------------------------------------*/
      /*  Now process the DBMS Access(es) for each step            */
      /*-----------------------------------------------------------*/            
      long StatAccess = mpActnPort->GetStatAccessCount();     
      if (StatAccess > 0)       
      {            
         for (int LocCnt = 1 ; LocCnt <= StatAccess ; LocCnt++) 
         {
            mpActnPort->PrimeStatAccess(LocCnt);                     
            switch (_ttoi(mpActnPort->GetStatAccessAccessType())) {
              case _T('S'):               
                ienum = OptimReportingContext::Cursor_Scan;
                break;
              case _T('P'):                
                ienum = OptimReportingContext::PK_lookup;
                break;
              case _T('F'):                
                ienum = OptimReportingContext::FK_lookup;
                break;
              default:
                ienum = ORC_DEFAULT_VALUE;
                break;
            }
            reportingStatistics->dbAccessType = ienum;         
            iValue = _ttoi(mpActnPort->GetStatAccessCompoundKeyCount());
            reportingStatistics->dbKeysPer = iValue;
            iValue = _ttoi(mpActnPort->GetStatAccessNumOpenCursors());
            reportingStatistics->dbOpenCursor  = iValue;
            iValue = _ttoi(mpActnPort->GetStatAccessNumRowsFetched());
            reportingStatistics->dbRowsFetched =iValue;
            iValue = _ttoi(mpActnPort->GetStatAccessNumRowsWrittenToXF());
            reportingStatistics->dbRowsWritten = iValue;           

            DWORD dwTimeInDBMSCalls  =  _ttoi(mpActnPort->GetStatAccessTimeInDBMSCalls());
            DWORD dwTimeInOtherCalls =  _ttoi(mpActnPort->GetStatAccessTimeInOtherCalls());
            DWORD dwTimeInArchiveActions = _ttoi(mpActnPort->GetStatAccessTimeInRowArchiveAct());
            DWORD dwTotalTime = dwTimeInDBMSCalls + dwTimeInOtherCalls + dwTimeInArchiveActions;

            // Percent time in DBMS reads.            
            csPercent.Format((const TCHAR *)csPercentageFormat, (int)((float)dwTimeInDBMSCalls/(float)dwTotalTime * 100));
            lValue = _ttol((const TCHAR *)csPercent);
            reportingStatistics->dbTimeInDB = lValue;

            // Percent time in DBMS archive actions.
            csPercent.Format((const TCHAR *)csPercentageFormat, (int)((float)dwTimeInArchiveActions/(float)dwTotalTime * 100) );               
            lValue = _ttol((const TCHAR *)csPercent);
            reportingStatistics->dbTimeInAct = lValue;

            // Process time
            DWORD Elapse;
            Elapse =_ttoi(mpActnPort->GetStatAccessElapsedTime());               
            reportingStatistics->dbProcessTime = Elapse;

            // Rows per second
            DWORD Fetched = _ttoi(mpActnPort->GetStatAccessNumRowsFetched());
            Fetched /= Elapse;
            reportingStatistics->dbRowsPerSecond = Fetched;

            if(Type == RDB_TERADATA)
            {
               ULONGLONG AverageTblDataSize = (ULONGLONG) _ttoi64(mpActnPort->GetStatAccessTableAvgDataSize());
               double AverageTblDataSizeInKB = (double) AverageTblDataSize / 1024;
               double AverageDataPerSec = AverageTblDataSizeInKB/Elapse; // In KB per Sec
               if(AverageDataPerSec == 0)
               {                  
                  reportingStatistics->dbDataPerSecond = ORC_DEFAULT_VALUE;
               }
               else
               {
                 reportingStatistics->dbDataPerSecond = AverageDataPerSec;
               }
            }
            else
            {
                reportingStatistics->dbDataPerSecond = ORC_DEFAULT_VALUE;
            }
         }
      }
      else
      {
         reportingStatistics->dbAccessType = ORC_DEFAULT_VALUE;
         reportingStatistics->dbKeysPer = ORC_DEFAULT_VALUE;         
         reportingStatistics->dbOpenCursor = ORC_DEFAULT_VALUE;
         reportingStatistics->dbRowsFetched = ORC_DEFAULT_VALUE;
         reportingStatistics->dbRowsWritten = ORC_DEFAULT_VALUE;
         reportingStatistics->dbDataPerSecond = ORC_DEFAULT_VALUE;
         reportingStatistics->dbProcessTime = ORC_DEFAULT_VALUE;
         reportingStatistics->dbTimeInAct = ORC_DEFAULT_VALUE;
         reportingStatistics->dbTimeInDB = ORC_DEFAULT_VALUE;

      }
      if (mpActnPort->GetFileCompression())
      {
         // For each table compression details                               
         wcscpy(reportingStatistics->compTableName, mpActnPort->GetCurTblName());     
         wcscpy(reportingStatistics->compRatio, mpActnPort->GetCurTblCompressRatio());
      }
      else
      {
        // For each table compression details     
        wcscpy(reportingStatistics->compTableName, ORC_EMPTY_LSTRING);     
        wcscpy(reportingStatistics->compRatio, ORC_EMPTY_LSTRING);            
      }       
   }
   reportingStatistics->pNext = NULL;
   
   return(TRUE);
}

/*---------------------------------------------------------------------------
* OptimReportingContext::GetSourceType
*--------------------------------------------------------------------------*/
int OptimReportingContext::GetSourceType( short sType, int source )
{
    RtTraceEntry(OptimReportingContext::GetSourceType( sType ) );

    CString csTempStr;
    CString csTableName;
    CString csTableMapName;
    int ienum;

    //Initialize string
    csTempStr.Empty();

    switch ( sType )
    {
    case ActnPort::SourceTypeInvalid:
        csTempStr = RtString(COMP_INVALIDTYPE);
        ienum = OptimReportingContext::SourceTypeInvalid;
        break;

    case ActnPort::SourceTypeExtractFile:
        csTempStr = RtString(COMP_EXTRACTFILE);
        ienum = OptimReportingContext::SourceTypeExtractFile;
        break;

    case ActnPort::SourceTypeDatabaseTable:
        // get table name
        if ( source == COMP_SOURCE_ONE )
            csTableName = mpActnPort->GetSource1Name();
        else
            csTableName = mpActnPort->GetSource2Name();
        csTempStr = RtFormatMsg( COMP_DATABASETABLE,_T("%s"), (const TCHAR *) csTableName );
        ienum = OptimReportingContext::SourceTypeDatabaseTable;
        break;

    case ActnPort::SourceTypeAccessDefinition:       
        csTempStr = RtString(RTKEY_ACCDEF_STR);
        ienum = OptimReportingContext::SourceTypeAccessDefinition;
        break;

    case ActnPort::SourceTypeAllTables:
        csTempStr = mpActnPort->GetSource2Name();
        csTempStr = RtFormatMsg( COMP_ALLTABLES, _T("%s"), (const TCHAR *) csTempStr ) ;
        ienum = OptimReportingContext::SourceTypeAllTables;
        break;

    }  // switch

    return ienum;

} 

/*-------------------------------------------------------------
*  OptimReportingContext::GetSource1Name ()
*--------------------------------------------------------------*/
CString OptimReportingContext::GetSource1Name ( short sType )
{
    RtTraceEntry(OptimReportingContext::GetSource1Name);

    CString csTempStr;

    csTempStr =  mpActnPort->GetSource1Name();

    switch ( sType )
    {

    case ActnPort::SourceTypeExtractFile:
        //Do nothing return Source name
        break;

    case ActnPort::SourceTypeAccessDefinition:
        // Access definition Local/ Named
        if ( csTempStr == _T("") )            
            csTempStr = RtString(RTDEFS_STRING_LOCAL);
        break;

    case ActnPort::SourceTypeInvalid:
    case ActnPort::SourceTypeDatabaseTable:
    case ActnPort::SourceTypeAllTables:
        // Display None        
        csTempStr = RtString( RTAGE_STRING_NONE );
        break;

    }  // switch

  return csTempStr;

}   

/*-------------------------------------------------------------
*  OptimReportingContext::GetSource2Name ()
*--------------------------------------------------------------*/
CString OptimReportingContext::GetSource2Name ( short sType )
{
    RtTraceEntry(OptimReportingContext::GetSource2Name);

    CString csTempStr;

    csTempStr =  mpActnPort->GetSource2Name();

    switch ( sType )
    {

    case ActnPort::SourceTypeExtractFile:
        //Do nothing return Source name
        break;

    case ActnPort::SourceTypeAccessDefinition:
        // Access definition Local/ Named
        if ( csTempStr == _T("") )
            
            csTempStr = RtString(RTDEFS_STRING_LOCAL);
        break;

    case ActnPort::SourceTypeInvalid:
    case ActnPort::SourceTypeDatabaseTable:
    case ActnPort::SourceTypeAllTables:
        // Display None        
        csTempStr = RtString( RTAGE_STRING_NONE );
        break;

    }  // switch

  return csTempStr;

}

/*---------------------------------------------------------------------------
 * OptimReportingContext:GetOptimDBInfo
 * Get DBInfo
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::GetOptimDBInfo()
{
	RtTraceEntry(OptimReportingContext::GetOptimDBInfo);

	RtDir OptDir;
   CString csOptDir = RtServGetPstDirectory();
   	
	// Use RtDir class instance to obtain Optim Dir DBMS type and ver
	OptDir.Load((const TCHAR*)csOptDir);
	switch(OptDir.GetDBMSType())
	{
	case RDB_ORACLE :
		mDBMSType = _T("Oracle");
		break;
	case RDB_DB2CS:
		mDBMSType = _T("DB2 LUW");
		break;
	case RDB_SQLSERVER:
		mDBMSType = _T("MS SQL Server");
		break;
	case RDB_INFORMIX:
		mDBMSType = _T("Informix");
		break;
	case RDB_SYBASE:
		mDBMSType = _T("Sybase");
		break;
	case RDB_TERADATA:
		mDBMSType = _T("Teradata");
		break;
	case RDB_NETEZZA:
		mDBMSType = _T("Netezza");
		break;
	default:
		mDBMSType = _T("");
	}

	// Get the DBMS version of Optim Dir
	GetOptimDBMSVer(_ttoi(OptDir.GetDBMSVer()));

   return TRUE;
}

/*----------------------------------------------------------------------------
 * OptimReportingContext::GetDBMSVerStr()
 * Get Database Server Version info
 *---------------------------------------------------------------------------*/
BOOL OptimReportingContext::GetOptimDBMSVer(int DBMSVer)
{
	RtTraceEntry(OptimReportingContext::GetDBMSVer);
	
	switch (DBMSVer)
	{
		// DB2 LUW
	case RDB_DBMSDLLVER_DB22_V0720:
		mDBMSVer = _T("7.2.x");
		break;
	case RDB_DBMSDLLVER_DB22_V0810:
		mDBMSVer = _T("8.1");
		break;
	case RDB_DBMSDLLVER_DB22_V0820:
		mDBMSVer = _T("8.2");
		break;
	case RDB_DBMSDLLVER_DB22_V0910:
		mDBMSVer = _T("9.1");
		break;
	case RDB_DBMSDLLVER_DB22_V0950:
		mDBMSVer = _T("9.5");
		break;
	case RDB_DBMSDLLVER_DB22_V0971:
		mDBMSVer = _T("9.7");
		break;
		//Oracle
	case RDB_DBMSDLLVER_ORACLE_V0800:
		mDBMSVer = _T("8.0.x");
		break;
	case RDB_DBMSDLLVER_ORACLE_V0815:
		mDBMSVer = _T("8.1.x");
		break;
	case RDB_DBMSDLLVER_ORACLE_V0900:
		mDBMSVer = _T("9.0.0");
		break;
	case RDB_DBMSDLLVER_ORACLE_V0920:
		mDBMSVer = _T("9.2.0");
		break;
	case RDB_DBMSDLLVER_ORACLE_V10G0:
		mDBMSVer = _T("10.1.0");
		break;
	case RDB_DBMSDLLVER_ORACLE_V1020:
		mDBMSVer = _T("10.2.0");
		break;
	case RDB_DBMSDLLVER_ORACLE_V1110:
		mDBMSVer = _T("11.1.0");
		break;
	case RDB_DBMSDLLVER_ORACLE_V1120:
		mDBMSVer = _T("11.2.0");
		break;
		//Sybase
	case RDB_DBMSDLLVER_SYBASE_V1000:
		mDBMSVer = _T("10.0.x");
		break;
	case RDB_DBMSDLLVER_SYBASE_V1100:
		mDBMSVer = _T("11.0.x");
		break;
	case RDB_DBMSDLLVER_SYBASE_V1109:
		mDBMSVer = _T("11.0.9");
		break;
	case RDB_DBMSDLLVER_SYBASE_V1205:
		mDBMSVer = _T("12.5.0");
		break;
	case RDB_DBMSDLLVER_SYBASE_V1500:
		mDBMSVer = _T("15.0");
		break;
		// Informix
	case RDB_DBMSDLLVER_INFORMIX_ODSV0723:
		mDBMSVer = _T("7.2.3");
		break;
	case RDB_DBMSDLLVER_INFORMIX_UDSV0912:
		mDBMSVer = _T("9.1.2");
		break;
		// SQL Server
	case RDB_DBMSDLLVER_SQLSERVER_V0700:
		mDBMSVer = _T("7.0");
		break;
	case RDB_DBMSDLLVER_SQLSERVER_V0800:
		mDBMSVer = _T("8.0");
		break;
	case RDB_DBMSDLLVER_SQLSERVER_V0900:
		mDBMSVer = _T("2005");
		break;
	case RDB_DBMSDLLVER_SQLSERVER_V1000:
		mDBMSVer = _T("2008");
		break;
	default:
		mDBMSVer = _T("Unknown");
		break;
	}

	return TRUE;
}

/*-----------------------------------------------------------------------
 * PSdynam
 *    NEC initialization entry
 *----------------------------------------------------------------------*/
extern "C" void PSdynamn(short mode)
{
   RtTraceEntry(PSdynamn);   
}